@@ -79,7 +79,7 @@ if ($selected == 0) {
 </tr>
 <tr>
   <td width="20%"><font size="+1"><b>Username</b></font></td>
-  <td width="80%"><input type="TEXT" size=20 name="USERNAME"
+  <td width="80%"><input type="TEXT" size="20" name="USERNAME"
 <?php
 if ($_REQUEST["USERNAME"] != "") {
   echo(" value=" . $_REQUEST["USERNAME"] . "></td>");
@@ -122,8 +122,8 @@ else {
 
 <p>
 <hr>
-<p><em><b>Copyright (c) 2003-2013 Distributed Systems Software.  All rights reserved.</b></em>
+<p><em><b>Copyright (c) 2003-2016 Distributed Systems Software.  All rights reserved.</b></em>
 <p>
-<em><b>$Id: login2.php 2627 2013-01-24 22:12:56Z brachman $</b></em>
+<em><b>$Id: login2.php 2873 2016-03-12 18:54:40Z brachman $</b></em>
 </body>
 </html>
