@@ -1,9 +1,9 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  *
- * $Id: auth.h 2594 2012-10-19 17:28:49Z brachman $
+ * $Id: auth.h 2925 2016-11-09 17:01:21Z brachman $
  */
 
 /*****************************************************************************
@@ -48,6 +48,8 @@
 #include <netinet/in.h>
 #include <sys/param.h>
 
+#include "dacs_crypto.h"
+
 /*
  * For Rijndael (AES), these three key sizes are valid:
  * 128 bits (16 bytes), 192 bits (24 bytes), and 256 bits (32 bytes)
@@ -76,7 +78,7 @@ enum {
    */
   AUTH_USE_RANDOMIZED_IV		    = 0,
 
-  /* Number of seconds to delay if authentication fails */
+  /* Number of seconds to delay if authentication fails.  May be zero. */
   AUTH_DEFAULT_FAIL_DELAY_SECS      = 8
 };
 
@@ -114,7 +116,8 @@ enum {
 #define AUTH_STYLE_RLINK      (1 << 16)
 #define AUTH_STYLE_SET_ROLES  (1 << 17)
 #define AUTH_STYLE_ADD_ROLES  (1 << 18)
-#define AUTH_STYLE_UNKNOWN    (1 << 19)
+#define AUTH_STYLE_TGMA		  (1 << 19)
+#define AUTH_STYLE_UNKNOWN    (1 << 20)
 
 typedef int Auth_style;
 
@@ -145,6 +148,8 @@ typedef int Auth_style;
 #define AUTH_ADMIN_USERNAME_PREFIX_STR	"*"
 #define AUTH_ADMIN_USERNAME_SUFFIX_STR	"*"
 
+#define AUTH_UA_HASH_DIGEST_NAME		"SHA1"
+
 /*
  * Credentials describe an authenticated user (i.e., a DACS identity).
  * Externally, an XML representation is encrypted and then encoded as text
@@ -210,9 +215,19 @@ typedef enum {
   COOKIE_NETSCAPE      = 0,
   COOKIE_EXT_NETSCAPE  = 1,
   COOKIE_RFC2109       = 2,
-  COOKIE_RFC2965       = 3
+  COOKIE_RFC2965       = 3,
+  COOKIE_RFC6265       = 4
 } Cookie_syntax;
 
+#define DACS_COOKIE_APP_NAME		"DACS"
+
+typedef struct Cookie_name_terminators {
+  char *app_end;				/* Terminates the application name. */
+  char *federation_end;			/* Terminates the federation name. */
+  char *jurisdiction_end;		/* Terminates the jurisdiction name. */
+  char *username_end;			/* Terminates the username, if required. */
+} Cookie_name_terminators;
+
 /* Netscape format plus RFC extensions */
 typedef struct Cookie {
   Cookie_syntax syntax;
@@ -346,6 +361,35 @@ typedef struct Cas_auth {
 
 /*****************************************************/
 
+/* Used by dacs_auth_tgma and local_tgma_authenticate. */
+
+/* Used to sign the hidden CGI variables on the login page. */
+#define TGMA_HMAC_DIGEST_NAME		"SHA3-256"
+
+/* Used to hash the Instance ID. */
+#define TGMA_IID_DIGEST_NAME		"SHA3-256"
+
+#define TGMA_SRP_DIGEST_NAME		"SHA3-256"
+
+enum {
+  TGMA_DEFAULT_WINDOW_SECS = 100,
+  TGMA_MIN_IID_LENGTH      = 4,
+  /* Number of random characters comprising an instance id. */
+  TGMA_INSTANCE_ID_NBYTES  = 4
+};
+
+typedef struct Tgma_auth {
+  char *iid;
+  char *clock_str;
+  char *nonce;
+  char *sig_str;
+  char *window_str;
+  char *addr_str;
+  int use_tcp;
+} Tgma_auth;
+
+/*****************************************************/
+
 /* Used by local_http_authenticate. */
 typedef struct Url_auth {
   Http_method method;
@@ -366,6 +410,48 @@ enum {
   PAMD_PAMD_TIMEOUT_SECS = 20
 };
 
+/*
+ * This will typically be the name of a file in /etc/pam.d containing
+ * the PAM policy to be used by local_unix_pam_auth().
+ * It can be overridden by an OPTION.
+ * The module requires the policy to prompt for exactly one thing: a password.
+ */
+#ifndef DEFAULT_PAM_SERVICE
+#define DEFAULT_PAM_SERVICE		"dacs-unix-auth"
+#endif
+
+/*
+ * This is the prompt we expect PAM to ask to be emitted, case-insensitively,
+ * for Unix password validation. Trailing spaces are always ignored; do not
+ * add them here.  You might need to change this if the prompt is in some
+ * other language, for instance.
+ *
+ * We do this check only to detect potential PAM or DACS misconfiguration.
+ * If this is set to the empty string, no check will be performed and the
+ * module will assume that everything is ok.
+ */
+#ifndef PAM_PASSWD_CONV_MSG
+#define PAM_PASSWD_CONV_MSG		"Password:"
+#endif
+
+/*
+ * For control of the Unix password type authentication via the
+ * local_unix_auth() and local_unix_pam_auth() methods.
+ */
+typedef enum {
+  AUTH_UNIX_WITHOUT_PAM = 0,	/* Do not use local_unix_pam_auth(). */
+  AUTH_UNIX_USE_PAM     = 1,	/* Only use local_unix_pam_auth(). */
+  AUTH_UNIX_TRY_PAM     = 2,	/* Try local_unix_pam_auth() if necessary. */
+  AUTH_UNIX_DEFAULT_PAM = AUTH_UNIX_TRY_PAM
+} Auth_unix_pam;
+
+/* Older versions of pam_strerror() didn't take the pam_handle_t argument. */
+#ifdef PAM_STRERROR_USES_PAMH
+#define PAM_STRERROR(HANDLE, ERR)		pam_strerror(HANDLE, ERR)
+#else
+#define PAM_STRERROR(HANDLE, ERR)		pam_strerror(ERR)
+#endif
+
 /*
  * This is (optionally) used to find the port used for pamd in /etc/services
  * and /etc/inetd.conf.
@@ -452,6 +538,12 @@ typedef enum {
   TOKEN_REQUIRE_PIN   = 1
 } Token_pin_mode;
 
+/*
+ * We would rather not assume that the HOTP counter length (currently defined
+ * to be eight bytes long) corresponds to a native data type.
+ * But it simplifies the code if we required it, so we'll worry about this
+ * if and when the counter length is extended.
+ */
 typedef unsigned int Auth_token_mode;
 #define AUTH_TOKEN_COUNTER_T unsigned long long
 typedef AUTH_TOKEN_COUNTER_T Auth_token_counter;
@@ -467,7 +559,7 @@ typedef struct Auth_token {
   char *digest_name;
   char *serial;					/* Device's serial number */
   unsigned char *counter;		/* Current counter value (HOTP only) */
-  unsigned char *key;			/* Device's secret key/seed */
+  unsigned char *key;			/* Device's secret key/seed, in binary */
   unsigned int keylen;			/* For internal use only */
   unsigned int time_step;		/* In seconds (TOTP only) */
   int drift;					/* In seconds (TOTP only) */
@@ -574,6 +666,7 @@ typedef enum {
   AUTH_MODULE_HTTP     = 10,
   AUTH_MODULE_TOKEN    = 11,
   AUTH_MODULE_PAM      = 12,
+  AUTH_MODULE_TGMA     = 13,
   AUTH_MODULE_UNKNOWN  = -1
 } Auth_module_id;
 
@@ -649,6 +742,9 @@ typedef struct Http_auth {
   Roles_module *roles_modules;
 } Http_auth;
 
+/* XXX may require code change where used. */
+#define AUTH_DIGEST_NONCE_DIGEST_NAME	"SHA1"
+
 /* A parsed WWW-Authentication response header */
 typedef struct Http_auth_www_authenticate {
   char *www_authenticate;	/* The header value; its parsed fields follow */
@@ -725,24 +821,32 @@ lookup_dbm_type(char *dbm_type)
 /*****************************************************/
 
 /*
- * This list may be extended but do not change the mapping because
- * the digest algorithm numbers are stored with the hashed passwords.
- * If you change a mapping, you will need to edit your DACS password files.
+ * If PASSWORD_DIGEST not configured, this is the default password hashing
+ * algorithm to use.
  */
-typedef enum {
-  PASSWD_ALG_CRYPT  = 0,
-  PASSWD_ALG_MD5    = 1,
-  PASSWD_ALG_SHA1   = 2,
-  PASSWD_ALG_SHA224 = 3,
-  PASSWD_ALG_SHA256 = 4,
-  PASSWD_ALG_SHA384 = 5,
-  PASSWD_ALG_SHA512 = 6,
-  PASSWD_ALG_NONE   = 999
-} Passwd_digest_alg;
-
-/* If not configured, the default password hashing algorithm to use. */
 #ifndef PASSWD_ALG_DEFAULT_NAME
-#define PASSWD_ALG_DEFAULT_NAME		"SHA1"
+#define PASSWD_ALG_DEFAULT_NAME		"SHA512"
+#endif
+
+/*
+ * Default parameters for password hashing functions.
+ */
+#define PASSWD_DEFAULT_PBKDF2_SHA1_COUNT	16384
+#define PASSWD_DEFAULT_PBKDF2_SHA1_DKLEN	32
+#define PASSWD_DEFAULT_PBKDF2_SHA256_COUNT	32768
+#define PASSWD_DEFAULT_PBKDF2_SHA256_DKLEN	32
+#define PASSWD_DEFAULT_PBKDF2_SHA512_COUNT	32768
+#define PASSWD_DEFAULT_PBKDF2_SHA512_DKLEN	64
+#define PASSWD_DEFAULT_SCRIPT_N				131072
+#define PASSWD_DEFAULT_SCRIPT_R				8
+#define PASSWD_DEFAULT_SCRIPT_P				1
+#define PASSWD_DEFAULT_SCRIPT_DKLEN			32
+#ifdef NOTDEF
+/* Alternatively: */
+#define PASSWD_DEFAULT_SCRIPT_N				32768
+#define PASSWD_DEFAULT_SCRIPT_R				8
+#define PASSWD_DEFAULT_SCRIPT_P				8
+#define PASSWD_DEFAULT_SCRIPT_DKLEN			64
 #endif
 
 enum {
@@ -786,6 +890,7 @@ typedef struct Pw_constraints {
  */
 #define PASSWD_SEP_CHAR		'|'
 
+#define PASSWD_ALG_SEP_STR	","
 /*
  * If this character is the first one in the stored digest entry,
  * the entry is in the disabled state and cannot be authenticated against.
@@ -794,9 +899,9 @@ typedef struct Pw_constraints {
 #define PASSWD_DISABLED_CHAR	'*'
 
 typedef enum {
-  PW_UNCHANGED = 0,
-  PW_DISABLED  = 1,
-  PW_ENABLED   = 2
+  PW_STATE_UNCHANGED = 0,
+  PW_STATE_DISABLED  = 1,
+  PW_STATE_ENABLED   = 2
 } Pw_state;
 
 /*
@@ -820,18 +925,27 @@ typedef enum {
   PW_OP_TEST_DISABLED  = (1 << 12),
   PW_OP_TEST_ENABLED   = (1 << 13),
   PW_OP_TEST_PRIVATE   = (1 << 14),
-  PW_OP_GET_DIGEST     = (1 << 15)
+  PW_OP_GET_DIGEST     = (1 << 15),
+  PW_OP_REGENERATE     = (1 << 16),
+  PW_OP_LONGLIST       = (1 << 17)
 } Pw_op;
 
-/* A password file entry in memory, as maintained by dacspasswd(1) etc. */
+/*
+ * A password file entry in memory, as maintained by dacspasswd(1) etc.
+ *
+ * For parameterized digest algorithms, parameters (and the algorithm identifier)
+ * are stored with each password entry, and the stored parameters are
+ * always used for validation, so changing the configuration will not invalidate
+ * previously stored passwords, only new or updated entries.
+ */
 typedef struct Pw_entry {
   char *username;		/* In memory only */
   Pw_state state;
-  Passwd_digest_alg alg;
+  Digest_desc *dd;		/* Secure hash algorithm used on the password. */
   char *salt;
-  char *digest;			/* The hash of a password and salt using alg */
-  char *stored_digest;	/* The formatted alg, salt, and digest */
-  char *entry;			/* The digest_str with encoded application data */
+  char *digest;			/* Hash of a password and salt using alg */
+  char *stored_digest;	/* Formatted alg, salt, and digest */
+  char *entry;			/* The stored_digest with encoded application data */
   Ds *private;			/* Arbitrary application data */
 } Pw_entry;
 
@@ -978,7 +1092,7 @@ typedef struct Ic_auth {
  */
 typedef struct Sic_entry {
   Ic_state state;
-  Passwd_digest_alg alg;
+  Digest_desc *desc;
   char *username;
   Ds *friendly_id;
   char *pubkey_modulus;
@@ -989,7 +1103,7 @@ typedef struct Sic_entry {
 
 typedef struct Mic_sts_auth {
   Ic_sts_authtype authtype;
-  Passwd_digest_alg alg;		/* For "passwd" type */
+  Digest_desc *desc;			/* For "passwd" type */
   char *digest;					/* For "passwd" type, optional */
   char *self_issued_ppid;		/* For "card" type */
   Ds *thumbprint;				/* For "cert" type */
@@ -1065,6 +1179,10 @@ extern char *non_auth_cookie_header;
 extern Cookie *non_auth_cookies;
 extern int debug_crypt;
 
+extern const Cookie_name_terminators default_cookie_name_terminators;
+extern Cookie_name_terminators current_cookie_name_terminators;
+
+
 /*****************************************************/
 
 typedef enum {
@@ -1139,6 +1257,9 @@ extern int is_dacs_admin_name(char *username);
 extern char *is_valid_valid_for(const char *valid_for);
 extern void set_valid_for(Credentials *credentials, char *valid_for);
 
+/* Password-style Message Digests */
+extern char *passwd_hash(Digest_desc *dd, char *passwd, char *salt);
+
 /* mkkey.c */
 extern Crypt_keys *crypt_keys_from_vfs(char *vfs_ref);
 extern Crypt_keys *crypt_keys_from_buf(char *keystr);
@@ -1185,6 +1306,11 @@ extern int make_set_auth_cookie_header(Credentials *credentials,
 									   char *lifetime, int cookie_only,
 									   char **cookie_buf);
 extern char *make_auth_cookie_name(Credentials *cr);
+extern char *make_cookie_name_app_prefix(const Cookie_name_terminators *term);
+extern char *
+	make_cookie_name_federation_prefix(const Cookie_name_terminators *term);
+extern Cookie_name_terminators *init_cookie_name_terminators(char *term_str);
+extern int set_cookie_name_terminators(Cookie_name_terminators *term);
 
 extern int get_cookies(Kwv *kwv, Cookie **cookies, unsigned *ncookies);
 extern void cookies_html(FILE *fp, Cookie *cookies);
@@ -1197,17 +1323,18 @@ extern Cookie *cookie_parse(char *cookie_value, int *is_dacs_cookie);
 extern int is_auth_cookie_name(char *cookie_name);
 extern int set_cookie_output_syntax(Cookie_syntax syntax);
 extern int configure_cookie_syntax(char *cookie_syntax, char **errmsg);
+extern char *make_cookie_name(Cookie_name_terminators *term,
+							  const char *app, const char *federation,
+							  char *jurisdiction, char *username,
+							  char *special);
+extern Cookie_name *parse_cookie_name(Cookie_name_terminators *term, char *name);
 
 extern char *get_passwd(Get_passwd_type type, void *arg);
 
-extern char *passwd_make_digest(Passwd_digest_alg alg, char *passwd,
-								Pw_entry *);
-extern int passwd_check_digest(Passwd_digest_alg alg, char *given_passwd,
-							   Pw_entry *);
-extern char *passwd_lookup_digest_name(Passwd_digest_alg alg);
-extern Passwd_digest_alg passwd_lookup_digest_algorithm(char *digest_name);
-extern int passwd_get_digest_algorithm(char **digest_name,
-									   Passwd_digest_alg *alg);
+extern char *passwd_make_digest(Digest_desc *dd, char *passwd, Pw_entry *);
+extern int passwd_check_digest(Digest_desc *dd, char *given_passwd, Pw_entry *);
+extern char *passwd_lookup_password_digest_name(Digest_desc *dd);
+extern Digest_desc *passwd_get_digest_algorithm(const char *digest_name);
 
 extern char *make_hash_from_credentials(Credentials *cr);
 
@@ -1341,7 +1468,7 @@ extern Pw_entry *pw_init_entry(Pw_entry *, char *);
 extern char *pw_make_entry(Pw_entry *pw);
 extern Pw_entry *pw_parse_entry(char *username, char *stored_digest);
 extern int pw_check_passwd(Vfs_handle *h, char *username, char *given_passwd,
-						   Passwd_digest_alg alg);
+						   Digest_desc *dd);
 extern Pw_entry *pw_read_entry(Vfs_handle *h, char *username);
 extern int pw_delete_entry(Vfs_handle *h, char *username);
 extern int pw_exists_entry(Vfs_handle *h, char *username);
@@ -1350,17 +1477,17 @@ extern Ds *pw_getdata_entry(Vfs_handle *h, char *username);
 extern int pw_test_entry(Vfs_handle *h, Pw_op op, char *username);
 extern Dsvec *pw_list_entries(Vfs_handle *h);
 extern int pw_add_entry(Vfs_handle *h, char *username, char *given_passwd,
-						Passwd_digest_alg alg, Pw_state new_state, Ds *data);
+						Digest_desc *dd, Pw_state new_state, Ds *data);
 extern int pw_reset_entry(Vfs_handle *h, char *username, char *given_passwd,
-						  Passwd_digest_alg alg, Pw_state new_state,
+						  Digest_desc *dd, Pw_state new_state,
 						  Pw_op data_op, Ds *data);
 extern int pw_rename_entry(Vfs_handle *h, char *old_username,
 						   char *new_username);
 extern int pw_replace_entry(Vfs_handle *h, char *username, char *given_passwd,
-							Passwd_digest_alg alg, Pw_state new_state,
+							Digest_desc *dd, Pw_state new_state,
 							Pw_op data_op, Ds *data);
 extern int pw_update_entry(Vfs_handle *h, char *username, char *given_passwd,
-						   Passwd_digest_alg alg, Pw_state new_state);
+						   Digest_desc *dd, Pw_state new_state);
 extern int pw_disable_entry(Vfs_handle *h, char *username);
 extern int pw_enable_entry(Vfs_handle *h, char *username);
 extern int pw_is_passwd_acceptable(char *passwd, char *constraints);
@@ -1385,7 +1512,7 @@ extern int mic_register_entry(Vfs_handle *h, char *identity, char *ppid,
 extern Ic_config *ic_config(void);
 extern Ic_auth *ic_lookup_entry(Vfs_handle *h, char *xmlToken, Ic_config *conf);
 extern Dsvec *ic_get_config_audience(void);
-extern int ic_get_digest_algorithm(char **digest_name, Passwd_digest_alg *alg);
+extern int ic_get_digest_algorithm(char **digest_name, Digest_desc *desc);
 extern Ic_sts_authtype ic_lookup_sts_authtype(char *authtype_str);
 extern int ic_lookup_sts_authtype_str(Ic_sts_authtype authtype,
 									  char **short_str, char **long_str);
@@ -1457,13 +1584,18 @@ extern int local_cas_auth(char *username, char *password, char *aux,
 						  Cas_auth *cas_auth);
 #endif
 
+#ifdef ENABLE_TGMA_AUTH
+extern int local_tgma_auth(char *username, char *password, char *aux,
+						  Tgma_auth *tgma_auth);
+#endif
+
 #ifdef ENABLE_GRID_AUTH
 extern int local_grid_auth(char *username, char *password, char *aux);
 #endif
 
 #ifdef ENABLE_HTTP_AUTH
 extern int local_http_auth(char *username, char *password, char *aux,
-						   Url_auth *url_auth);
+						   Url_auth *url_auth, Auth_reply **auth_reply);
 #endif
 
 #ifdef ENABLE_INFOCARD_AUTH
@@ -1505,6 +1637,8 @@ extern int local_token_auth(char *username, char *password, char *aux,
 
 #ifdef ENABLE_UNIX_AUTH
 extern int local_unix_auth(char *username, char *password, char *aux);
+extern int local_unix_pam_auth(char *pam_service, char *username,
+							   char *password, char *aux);
 #endif
 
 /* Local roles modules */
@@ -1520,6 +1654,26 @@ extern Dsvec *user_info_load(char *jurisdiction);
 extern Dsvec *user_info_active(Dsvec *dsv, char *ident);
 extern Dsvec *user_info_last_auth(Dsvec *dsv, char *ident);
 
+#ifdef ENABLE_TGMA_AUTH
+extern void tgma_format_form_hidden(Ds *ds, char *name, char *value);
+extern Ds *tgma_format_form_vars(char *username, char *jurisdiction,
+								 Tgma_auth *tgma_auth);
+extern char *tgma_form_hmac(Ds *ds);
+extern char *tgma_iid_hash(const char *iid);
+extern int tgma_tcp_read(int sd, unsigned char *buf, size_t *buflen,
+						 unsigned int timeout_secs);
+extern int tgma_tcp_write(int sd, unsigned char *buf, size_t buflen);
+extern int tgma_send(int sd,  struct sockaddr_in *to, unsigned char *datap,
+					 size_t datalen);
+extern int tgma_recv(int sd, int max_secs, unsigned char *bufp, size_t *buflen,
+					 struct sockaddr *from, socklen_t *fromlen);
+extern char *tgma_hmac_result(Ds *hkey, Ds *str);
+extern char *tgma_make_iid(void);
+extern int tgma_request(struct sockaddr_in *tgma_addr, char *username,
+						char *jurisdiction, char *iid, int use_tcp,
+						unsigned int tgma_window_secs);
+#endif
+
 #ifdef ENABLE_USER_INFO
 extern void user_info_authenticate(Credentials *cr);
 extern void user_info_signout(Credentials *cr);
