@@ -1,9 +1,9 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2014
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  *
- * $Id: conf.h 2594 2012-10-19 17:28:49Z brachman $
+ * $Id: conf.h 2712 2014-09-04 23:16:19Z brachman $
  */
 
 /*****************************************************************************
@@ -195,7 +195,8 @@ enum {
   CONF_REGISTER_NICKNAMES                     = 126,
   CONF_REGISTER_USE_QUESTION                  = 127,
   CONF_TOKEN_REQUIRES_PIN                     = 128,
-  CONF_TOKEN_HOTP_ACCEPT_WINDOW               = 129
+  CONF_TOKEN_HOTP_ACCEPT_WINDOW               = 129,
+  CONF_COOKIE_NAME_TERMINATORS                = 130
 };
 
 enum {
