@@ -1,9 +1,9 @@
 /*
- * Copyright (c) 2003-2013
+ * Copyright (c) 2003-2015
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  *
- * $Id: ds.h 2640 2013-02-27 19:41:06Z brachman $
+ * $Id: ds.h 2905 2016-07-05 14:54:12Z brachman $
  */
 
 /*****************************************************************************
@@ -102,7 +102,9 @@ typedef struct Ds {
 
 enum ds_prompt_flag {
   DS_PROMPT_ECHO    = 0,
-  DS_PROMPT_NOECHO  = 1
+  DS_PROMPT_NOECHO  = 1,
+  DS_PROMPT_HIDE    = 2,	/* Not implemented. */
+  DS_PROMPT_MAYECHO = 3		/* Not implemented. */
 };
 
 /*
@@ -218,12 +220,28 @@ extern Ds *ds_agetf(Ds *ds, FILE *fp);
 extern char *ds_prompt(Ds *ds, char *prompt, unsigned int flag);
 extern Ds *ds_subst(Ds *ds, unsigned char ch_src, unsigned char ch_rpl);
 
-extern char *ds_xprintf(const char *fmt, ...);
-extern char *ds_vxprintf(const char *fmt, va_list ap);
-extern int ds_sprintf(Ds *ds, unsigned int offset, const char *fmt, ...);
+#ifndef NOTDEF
+extern int ds_asprintf(Ds *ds, const char *fmt, ...)
+  __attribute__ ((format (printf, 2, 0)));
+extern int ds_psprintf(Ds *ds, const char *fmt, ...)
+  __attribute__ ((format (printf, 2, 0)));
+extern int ds_sprintf(Ds *ds, unsigned int offset, const char *fmt, ...)
+  __attribute__ ((format (printf, 3, 0)));
+extern char *ds_xprintf(const char *fmt, ...)
+  __attribute__ ((format (printf, 1, 2)));
+extern int ds_vasprintf(Ds *ds, const char *fmt, va_list ap)
+  __attribute__ ((format (printf, 2, 0)));
+extern char *ds_vxprintf(const char *fmt, va_list ap)
+  __attribute__ ((format (printf, 1, 0)));
+#else
 extern int ds_asprintf(Ds *ds, const char *fmt, ...);
-extern int ds_vasprintf(Ds *ds, const char *fmt, va_list ap);
 extern int ds_psprintf(Ds *ds, const char *fmt, ...);
+extern int ds_sprintf(Ds *ds, unsigned int offset, const char *fmt, ...);
+extern int ds_vasprintf(Ds *ds, const char *fmt, va_list ap);
+extern char *ds_vxprintf(const char *fmt, va_list ap);
+extern char *ds_xprintf(const char *fmt, ...);
+#endif
+
 
 extern FILE *ds_fopen_secure(const char *path, const char *mode, size_t size);
 extern int ds_fclose(FILE *fp);
@@ -315,11 +333,13 @@ dsvec_size(Dsvec *dsv)
 }
 
 typedef struct Mkargv {
-  int keepq;
-  int keepws;
-  char *ifs;
-  char *startq;
-  char *endq;
+  int keepq;		/* If non-zero, include the start/end quote character with the element. */
+  int keepws;		/* If non-zero, include whitespace that precedes an element with the element. */
+  char *ifs;		/* Any character in this string is a field separator character. */
+  char *startq;		/* Any character in this string is a starting quote character. */
+  char *endq;		/* Each character is an ending quote character, but only when it matches the
+					 * corresponding starting quote character.
+					 */
 } Mkargv;
 
 /* For dsvec_select() */
@@ -367,6 +387,7 @@ extern int dsvec_append(Dsvec *primary, Dsvec *secondary);
 extern int dsvec_set(Dsvec *odsv, void *base, size_t size, int n);
 extern int dsvec_add_obj(Dsvec *dsv);
 extern Dsvec *dsvec_list(Dsvec *odsv, char **list);
+extern Dsvec *dsvec_csv(char *str, char *ifs, char *startq, char *endq, int include_ws);
 
 extern void dsvec_sort(Dsvec *dsv, int (*compar)(const void *, const void *));
 extern int dsvec_compar_string(const void *ap, const void *bp);
@@ -409,7 +430,8 @@ extern Dsvec *dsvec_lines(Dsvec *dsv, char *buf);
 extern Dsvec *dsvec_load_lines(Dsvec *dsv, char *pathname);
 extern int dsvec_streq(Dsvec *dsv1, Dsvec *dsv2);
 extern int dsvec_strneq(Dsvec *dsv1, Dsvec *dsv2, size_t len);
-extern Dsvec *dsvec_strlist(Dsvec *odsv, char *str_spec);
+extern Dsvec *dsvec_strlist(Dsvec *odsv, const char *str_spec);
+extern void dsvec_str_emit(FILE *ofp, Dsvec *dsv);
 
 extern int dsvec_is_subset(Dsvec *dsv, Dsvec *dsv_subset,
 						   int (*compar)(const void *, const void *));
