@@ -1,9 +1,9 @@
 /*
- * Copyright (c) 2003-2013
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  *
- * $Id: str.h 2639 2013-02-27 19:40:13Z brachman $
+ * $Id: str.h 2910 2016-10-18 19:46:38Z brachman $
  */
 
 /*****************************************************************************
@@ -165,11 +165,39 @@ is_octdigit(int ch)
   return(ch >= '0' && ch <= '7');
 }
 
+/* Assumes CH is known to be an octal digit. */
 static inline MAYBE_UNUSED int
 octdigit_val(int ch)
 {
 
-  return(ch - '0');
+  if (ch >= '0' && ch <= '9')
+	return(ch - '0');
+
+  return(-1);
+}
+
+static inline MAYBE_UNUSED int
+is_hexdigit(int ch)
+{
+
+  return((ch >= '0' && ch <= '9')
+		 || (ch >= 'a' && ch <= 'f')
+		 || (ch >= 'A' && ch <= 'F'));
+}
+
+/* Assumes CH is known to be a hex digit. */
+static inline MAYBE_UNUSED int
+hexdigit_val(int ch)
+{
+
+  if (ch >= '0' && ch <= '9')
+	return(ch - '0');
+  if (ch >= 'a' && ch <= 'f')
+	return(ch - 'a' + 10);
+  if (ch >= 'A' && ch <= 'F')
+	return(ch - 'A' + 10);
+
+  return(-1);
 }
 
 /* For strbtohex() */
@@ -193,6 +221,7 @@ typedef struct Strfields {
   char **argv;
 } Strfields;
 
+#include <math.h>
 #include <inttypes.h>
 
 typedef enum {
@@ -230,6 +259,15 @@ typedef enum {
   STRNUM_IMAX_T    = 17, STRNUM_INTMAX_T  = 17,
 #define STRNUM_UIMAX_T_TYPE		uintmax_t
   STRNUM_UIMAX_T   = 18, STRNUM_UINTMAX_T = 18,
+#define STRNUM_I32_T_TYPE		int32_t
+  STRNUM_I32_T     = 19, STRNUM_INT32_T   = 19,
+#define STRNUM_I64_T_TYPE		int64_t
+  STRNUM_I64_T     = 20, STRNUM_INT64_T   = 20,
+#define STRNUM_UI32_T_TYPE		uint32_t
+  STRNUM_UI32_T    = 21, STRNUM_UINT32_T  = 21,
+#define STRNUM_UI64_T_TYPE		uint64_t
+  STRNUM_UI64_T    = 22, STRNUM_UINT64_T  = 22,
+  STRNUM_F         = 23, STRNUM_D         = 24,
   STRNUM_ERR       = -1
 } Strnum;
 
@@ -307,14 +345,27 @@ extern char *strprefix(const char *p, const char *prefix);
 extern char *strcaseprefix(const char *p, const char *prefix);
 extern int strncaseprefix(char *prefix, char *word, size_t match_len);
 extern Ds *strfold(char *str);
-extern int is_digit_string(char *str);
-extern int is_signed_digit_string(char *str);
+extern int is_alpha_string(const char *str);
+extern int is_digit_string(const char *str);
+extern int is_signed_digit_string(const char *str);
 extern int is_strclass(char *str, int allowable, ...);
 
+extern unsigned char *stra32b(const char *inp, unsigned char **outp,
+							  unsigned int *nbytes);
+extern unsigned char *strdec32(const char *inp);
+extern unsigned int strba32(const unsigned char *inp, unsigned int nbytes,
+							int suppress_padding, char **outp);
+extern char *strbenc32(const unsigned char *str, unsigned int len);
+extern char *strbenc32_np(const unsigned char *str, unsigned int len);
+extern char *strenc32(char *str);
+
+extern Ds *ds_strba64(Ds *ods, const unsigned char *inp, unsigned int nbytes);
+extern Ds *ds_stra64b(Ds *ods, const char *inp);
 extern unsigned int strba64(const unsigned char *inp,
 							unsigned int nbytes, char **outp);
 extern unsigned char *stra64b(const char *inp,
 							  unsigned char **outp, unsigned int *nbytes);
+extern Ds *ds_strba85(Ds *ods, const unsigned char *inp, unsigned int nbytes);
 extern unsigned int strba85(const unsigned char *inp, unsigned int nbytes,
 							char **outp);
 extern unsigned char *stra85b(const char *inp, unsigned char **outp,
@@ -349,6 +400,7 @@ extern char *strpack(Strpack *desc, const char *delimit);
 extern int strunpack(const char *str, const char *delimit, int limit,
 					 Strpack *desc);
 
+extern int memcaseeq(const void *b1, const void *b2, size_t len);
 extern void memzap(void *s, size_t len);
 extern void memzapb(void *s, int val, size_t len);
 extern void *memdupn(void *, size_t len);
@@ -356,8 +408,10 @@ extern void *memdupn(void *, size_t len);
 extern void envzap(char *name);
 extern char *env_parse(char *env_string, char **env_name, char **env_val);
 extern char *strcpyn(char *dst, const char *src, size_t n);
-extern char *strtolower(char *);
-extern char *strtoupper(char *);
+extern char *strtolower(const char *);
+extern char *strlowerbuf(char *buf, char *str);
+extern char *strtoupper(const char *);
+extern char *strcapitalize(Ds *ods, char *str);
 extern char *strquote(char *str, char *qchars);
 extern char *strdequote(char *p);
 extern char *strextract(char *str, char *start_q, char *end_q);
@@ -391,6 +445,7 @@ extern char *strbtohex(unsigned char *buf, size_t len, int flags);
 extern Ds *strsubst(char *str, Strsubst *tab);
 extern char *strunique(char *str);
 extern char *strelide(char *str, char *del);
+extern Ds *strelidews(Ds *nds, Ds *ods);
 extern char *strchop(char *str, char *del);
 
 extern char *strchrdup(const char *s, int stop_ch, char **endp);
