@@ -1,9 +1,9 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  *
- * $Id: var.h 2594 2012-10-19 17:28:49Z brachman $
+ * $Id: var.h 2910 2016-10-18 19:46:38Z brachman $
  */
 
 #ifndef _VAR_H_
@@ -50,16 +50,17 @@ typedef struct Var_ref {
 
 typedef struct Var_ns Var_ns;
 
-/* Bit flags */
+/* Namespace bit flags */
 #define VAR_NS_DEFAULT			0			/* No flags */
-#define VAR_NS_READONLY			(1 << 0)
-#define VAR_NS_RESERVED			(1 << 1)
-#define VAR_NS_TEMPORARY		(1 << 2)
+#define VAR_NS_READONLY			(1 << 0)	/* User cannot modify variables. */
+#define VAR_NS_RESERVED			(1 << 1)	/* Namespace name is reserved. */
+#define VAR_NS_TEMPORARY		(1 << 2)	/* Namespace is temporary. */
+#define VAR_NS_STREAM			(1 << 3)	/* Contains stream context. */
 
 struct Var_ns {
-  char *ns;
-  Kwv *kwv;
-  unsigned int flags;
+  char *ns;				/* Name of the namespace. */
+  Kwv *kwv;				/* Its variables and their values. */
+  unsigned int flags;	/* Bit flags. */
   Var_ns *prev;
   Var_ns *next;
 };
