@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2015
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -10,10 +10,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2015\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: acs_token.c 2586 2012-03-15 16:21:40Z brachman $";
+  "$Id: acs_token.c 2808 2015-07-22 21:39:32Z brachman $";
 #endif
 
 #include "auth.h"
@@ -26,7 +26,9 @@ make_token_cookie_name(Acs_token *token)
 {
   char *name;
 
-  name = ds_xprintf("DACS:%s::TOKEN-%s", token->issued_by, token->unique);
+  name = make_cookie_name(NULL, DACS_COOKIE_APP_NAME,
+						  token->issued_by_federation,
+						  token->issued_by_jurisdiction, NULL, token->unique);
 
   return(name);
 }
@@ -49,7 +51,8 @@ make_token_value(char *str, char **value)
   unsigned char outbuf[256];
 
   ck = crypt_keys_from_vfs(ITEM_TYPE_JURISDICTION_KEYS);
-  conf = crypto_hmac_open("SHA1", ck->hmac_key, CRYPTO_HMAC_KEY_LENGTH);
+  conf = crypto_hmac_open(ACS_TOKEN_HMAC_DIGEST_NAME, ck->hmac_key,
+						  CRYPTO_HMAC_KEY_LENGTH);
   crypt_keys_free(ck);
   crypto_hmac_hash(conf, (unsigned char *) str, strlen(str));
   crypto_hmac_close(conf, outbuf, NULL);
@@ -93,7 +96,8 @@ flatten_token(Acs_token *token, Kwv *token_kwv, char **value,
 	kwv = kwv_init(10);
 	kwv->dup_mode = KWV_ALLOW_DUPS;
 
-	kwv_add(kwv, "ISSUED_BY", token->issued_by);
+	kwv_add(kwv, "ISSUED_BY_FEDERATION", token->issued_by_federation);
+	kwv_add(kwv, "ISSUED_BY_JURISDICTION", token->issued_by_jurisdiction);
 	kwv_add(kwv, "URL_PATH", token->url_path);
 	if (token->identity != NULL)
 	  kwv_add(kwv, "IDENTITY", token->identity);
@@ -117,7 +121,8 @@ flatten_token(Acs_token *token, Kwv *token_kwv, char **value,
 	kwv = token_kwv;
 
   p = kwv_str(kwv);
-  *valuelen = strba64((unsigned char *) p, strlen(p) + 1, value);
+  /* Encode the null terminator, too. */
+  *valuelen = strba64((unsigned char *) p, strlen(p) + 1, value) + 1;
 }
 
 /*
@@ -351,7 +356,8 @@ acs_token_create(Credentials *selected, Acs_result *result, char **cookie_buf)
 	return(0);
   }
 
-  token.issued_by = dacs_current_jurisdiction();
+  token.issued_by_federation = conf_val(CONF_FEDERATION_NAME);
+  token.issued_by_jurisdiction = conf_val(CONF_JURISDICTION_NAME);
 
   /*
    * Note: only the path of the matching Service is significant.
@@ -456,7 +462,8 @@ acs_token_cleanup(void)
 	  do_delete(h, keys[i]);
 	  continue;
 	}
-	if (kwv_lookup_value(token_kwv, "ISSUED_BY") == NULL) {
+	if (kwv_lookup_value(token_kwv, "ISSUED_BY_FEDERATION") == NULL
+		|| kwv_lookup_value(token_kwv, "ISSUED_BY_JURISDICTION") == NULL) {
 	  do_delete(h, keys[i]);
 	  continue;
 	}
@@ -558,8 +565,10 @@ acs_token_grants(Credentials *selected, char *uri, Cookie *cookies,
 	   * XXX Should we check that the token is coming from the same IP address
 	   * as when it was issued?  This is a problematic test...
 	   */
-	  if ((v = kwv_lookup_value(token_kwv, "ISSUED_BY")) == NULL
-		  || !streq(v, dacs_current_jurisdiction()))
+	  if ((v = kwv_lookup_value(token_kwv, "ISSUED_BY_FEDERATION")) == NULL
+		  || !streq(v, conf_val(CONF_FEDERATION_NAME))
+		  || (v = kwv_lookup_value(token_kwv, "ISSUED_BY_JURISDICTION")) == NULL
+		  || !streq(v, conf_val(CONF_JURISDICTION_NAME)))
 		continue;
 	  if ((v = kwv_lookup_value(token_kwv, "UNIQUE")) == NULL
 		  || !streq(v, key))
