@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2013
+ * Copyright (c) 2003-2015
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -43,10 +43,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2013\n\
+"Copyright (c) 2003-2015\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: conf.c 2642 2013-02-27 19:48:45Z brachman $";
+  "$Id: conf.c 2827 2015-07-23 23:37:16Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -211,6 +211,7 @@ Kwv_vartab conf_vartab[] = {
   CONF_DIRECTIVE(REGISTER_USE_QUESTION,                   KWV_OPTIONAL1),
   CONF_DIRECTIVE(TOKEN_REQUIRES_PIN,                      KWV_OPTIONAL1),
   CONF_DIRECTIVE(TOKEN_HOTP_ACCEPT_WINDOW,                KWV_OPTIONAL1),
+  CONF_DIRECTIVE(COOKIE_NAME_TERMINATORS,                 KWV_OPTIONAL1),
   CONF_NULL_DIRECTIVE()
 };
 
@@ -1111,7 +1112,7 @@ conf_eval(char *name, char *val, const Kwv_conf *conf, void *eval_arg,
 	return(-1);
   }
 
-  if (is_undef(&result.value)) {
+  if (st == ACS_EXPR_UNDEF) {
 	log_msg((LOG_TRACE_LEVEL, "Directive has been undefined: '%s'", name));
 	return(0);
   }
@@ -1312,9 +1313,8 @@ conf_resolve(Kwv *kwv, Site *site, Kwv_vartab *conf_vartab,
 	  return(NULL);
 	}
 
-	if (is_undef(&result.value)) {
-	  log_msg((LOG_TRACE_LEVEL, "Directive has been undefined: '%s'",
-			   pair->name));
+	if (st == ACS_EXPR_UNDEF) {
+	  log_msg((LOG_TRACE_LEVEL, "Directive has been undefined: '%s'", pair->name));
 	  kwv_delete(xkwv, pair->name);
 	  continue;
 	}
