@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2014
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -22,10 +22,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2014\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: cred.c 2594 2012-10-19 17:28:49Z brachman $";
+  "$Id: cred.c 2756 2014-12-31 17:58:38Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -1021,6 +1021,7 @@ dacs_usage(void)
   fprintf(stderr, "   -dd dir\n");
   fprintf(stderr, "   -ll log_level\n");
   fprintf(stderr, "   -v\n");
+  fprintf(stderr, "   --version\n");
   fprintf(stderr, " op:\n");
   fprintf(stderr,
 		  "   auth [-p | -pf file] [-aux aux] [-ccf file] [-caf file] [-s] JURISDICTION:username [auth-URL]\n");
@@ -1080,6 +1081,10 @@ dacscred_main(int argc, char **argv, int do_init, void *main_out)
 	}
 	else if (streq(argv[i], "-v"))
 	  vflag++;
+	else if (strcaseeq(argv[i], "--version")) {
+	  fprintf(stderr, "%s\n", dacs_version_string());
+	  exit(0);
+	}
 	else
 	  break;
   }
