@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -10,14 +10,45 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2016\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: dacs.c 2602 2012-10-19 17:50:16Z brachman $";
+  "$Id: dacs.c 2876 2016-05-16 23:00:00Z brachman $";
 #endif
 
 #include "dacs.h"
 
+static int
+show_digests(int argc, char **argv, int do_init, void *main_out)
+{
+
+  if (argc == 2)
+	crypto_show_digest_info(stdout, argv[1]);
+  else
+	crypto_show_digest_info(stdout, NULL);
+
+  return(0);
+}
+
+static int
+check_digest(int argc, char **argv, int do_init, void *main_out)
+{
+  Digest_desc *dd;
+
+  if (argc != 2) {
+	printf("Digest spec required\n");
+	return(-1);
+  }
+
+  if ((dd = passwd_get_digest_algorithm(argv[1])) == NULL) {
+	printf("Digest spec is invalid: \"%s\"\n", argv[1]);
+	return(-1);
+  }
+
+  printf("%s: OK\n", dd->formatted_desc);
+  return(0);
+}
+
 typedef struct Commands {
   char *name;
   int (*func)(int argc, char **argv, int do_init, void *main_out);
@@ -28,6 +59,7 @@ typedef struct Commands {
  * canonical name first.
  */
 static Commands commands[] = {
+  { "checkdigest",             check_digest },
   { "dacsacl,aclcheck",        dacsacl_main },
   { "dacsauth",                dacsauth_main },
   { "dacsgrid,auth_grid",      dacsgrid_main },
@@ -44,6 +76,7 @@ static Commands commands[] = {
   { "dacsversion,version",     dacsversion_main },
   { "dacsvfs,vfs",             dacsvfs_main },
   { "dacshttp",                dacshttp_main },
+  { "digests",                 show_digests },
   { "license",                 show_license },
   { "sslclient",               sslclient_main },
   { NULL,                      NULL }
@@ -55,7 +88,7 @@ dacs_usage(void)
   int i;
 
   fprintf(stderr,
-		  "Usage: dacs [-v|--verbose] [--version] [--license] [--dumpenv] [<command> <command-args>]\n");
+		  "Usage: dacs [-v|--verbose] [--version] [--checkdigest digest-spec] [--digests] [--license] [--dumpenv] [<command> <command-args>]\n");
   fprintf(stderr, "Where <command> is one of:\n");
   for (i = 0; commands[i].name != NULL; i++) {
 	fprintf(stderr, "  %s\n", commands[i].name);
@@ -81,6 +114,25 @@ main(int argc, char **argv)
 	  envdump();
 	  exit(0);
 	}
+	else if (streq(argv[i], "--checkdigest")) {
+	  if (argv[i + 1] == NULL) {
+		dacs_usage();
+		/*NOTREACHED*/
+	  }
+
+	  if (check_digest(2, argv + 1, 0, NULL) == -1)
+		exit(1);
+
+	  exit(0);
+	}
+	else if (streq(argv[i], "--digest") || streq(argv[i], "--digests")) {
+	  if (argv[i + 1] != NULL) {
+		show_digests(2, argv + 1, 0, NULL);
+	  }
+	  else
+		show_digests(0, NULL, 1, NULL);
+	  exit(0);
+	}
 	else if (streq(argv[i], "--license")) {
 	  show_license(0, NULL, 1, NULL);
 	  exit(0);
