@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2015
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -41,10 +41,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2015\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: email.c 2586 2012-03-15 16:21:40Z brachman $";
+  "$Id: email.c 2766 2015-01-05 17:38:10Z brachman $";
 #endif
 
 #include "dacs_config.h"
@@ -952,6 +952,7 @@ test_transformed_email(char *to, char *from, char *subject,
   int st;
   char *url;
   Dsvec *headers, *mailer_args;
+  Email_dkim *dkim;
   Email_header *eh;
   Email_message *em;
   Kwv *kwv;
@@ -971,9 +972,10 @@ test_transformed_email(char *to, char *from, char *subject,
   kwv = kwv_init(8);
   kwv_add(kwv, "lifetime", make_approx_relative_date(60));
   kwv_add(kwv, "url", url);
+  dkim = email_dkim(CANON_SIMPLE);
   em = email_create_transformed(SENDMAIL_PROG, mailer_args, headers,
 								"multipart/alternative",
-								template_file, kwv, CANON_SIMPLE);
+								template_file, kwv, dkim);
   if (em != NULL)
 	st = email_send(em);
   else
@@ -1006,6 +1008,7 @@ usage(void)
   fprintf(stderr, "{-t|--to} <addr>:            recipient (repeatable)\n");
   fprintf(stderr, "-transform:                  filter the body\n");
   fprintf(stderr, "{-v|--verbose}:              verbose output\n");
+  fprintf(stderr, "--version:                   print version and exit\n");
   fprintf(stderr, "-var <name> <value>:         create a variable\n");
   fprintf(stderr, "\n");
   fprintf(stderr, "If no source for the body is specified, or if the\n");
@@ -1228,6 +1231,10 @@ dacsemail_main(int argc, char **argv, int do_init, void *main_out)
 	  do_transform++;
 	else if (streq(argv[i], "-v") || streq(argv[i], "--verbose"))
 	  verbose_flag++;
+	else if (strcaseeq(argv[i], "--version")) {
+	  fprintf(stderr, "%s\n", dacs_version_string());
+	  exit(0);
+	}
 	else {
 	  usage();
 	  /*NOTREACHED*/
