@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -39,10 +39,11 @@
 
 /*
  * Frame memory - context-based memory management
+ *
  * In short, this package keeps track of every memory allocation so that
  * all of them can conveniently be freed by the application, eliminating
  * leaks and potentially increasing security.
- * It might be viewed as being part way to a garbage collection approach.
+ * It might be thought of as a semi-automatic garbage collection method.
  *
  * A variety of different kinds of memory allocations and management modes
  * can be selected.
@@ -53,6 +54,8 @@
  *    in "unit_free" mode or release can be restricted to being applied to all
  *    allocations on a particular frame at once.
  *
+ * It can also be enabled/disabled at will (using global disable_frame_memory).
+ *
  * calloc(3), realloc(3), and reallocf(3) functionality is provided, but
  * not valloc(3).
  *
@@ -86,13 +89,16 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2016\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: frame.c 2594 2012-10-19 17:28:49Z brachman $";
+  "$Id: frame.c 2876 2016-05-16 23:00:00Z brachman $";
 #endif
 
 /*
+ * Memory allocations are associated with at most one "frame".
+ * A frame exists on its own or on a stack (LIFO) of frames.
+ *
  * Frames     Memory Chunk Allocation List
  *  ----         -----           -----
  * | F0 | ----> | MC0 |  ---->  | MC1 | ---->
@@ -505,9 +511,9 @@ static inline MemChunkAlloc *
 mem_mem_to_mca(void *ptr)
 {
   size_t diff;
-  MemChunkAlloc *mca;
+  MemChunkAlloc *mca, mca_bogus;
 
-  diff = (size_t) ((Fm_ui8 *) &mca->bogus - (Fm_ui8 *) &mca->size);
+  diff = (size_t) ((Fm_ui8 *) &mca_bogus.bogus - (Fm_ui8 *) &mca_bogus.size);
   mca = (MemChunkAlloc *) ((Fm_ui8 *) ptr - diff);
 
   return(mca);
@@ -517,9 +523,9 @@ static inline Fm_ui32
 mem_mca_data_length(size_t total_size)
 {
   size_t diff;
-  MemChunkAlloc *mca;
+  MemChunkAlloc *mca, mca_bogus;
 
-  diff = (size_t) ((Fm_ui8 *) &mca->bogus - (Fm_ui8 *) &mca->size);
+  diff = (size_t) ((Fm_ui8 *) &mca_bogus.bogus - (Fm_ui8 *) &mca_bogus.size);
   return(total_size - diff);
 }
 
