@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2015
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -43,10 +43,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2015\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: grouplib.c 2545 2012-01-11 19:45:11Z brachman $";
+  "$Id: grouplib.c 2861 2015-12-28 22:17:08Z brachman $";
 #endif
 
 #include "group.h"
@@ -370,9 +370,9 @@ make_group_names_from_role_str(char *jurisdiction, char *role_str)
 	group_names[n] = ALLOC(Group_name);
 	group_names[n]->jurisdiction = strdup(jurisdiction);
 	if (prefix == NULL)
-	  group_names[n]->username = ds_xprintf("%.*s", (q - p), p);
+	  group_names[n]->username = ds_xprintf("%.*s", (int) (q - p), p);
 	else
-	  group_names[n]->username = ds_xprintf("%s/%.*s", prefix, (q - p), p);
+	  group_names[n]->username = ds_xprintf("%s/%.*s", prefix, (int) (q - p), p);
 
 	n++;
 	if (*q == '/')
