@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2013
+ * Copyright (c) 2003-2015
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -49,10 +49,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2013\n\
+"Copyright (c) 2003-2015\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: http.c 2642 2013-02-27 19:48:45Z brachman $";
+  "$Id: http.c 2861 2015-12-28 22:17:08Z brachman $";
 #endif
 
 #include "dacs_config.h"
@@ -218,7 +218,7 @@ build_uri_path(Uri *uri)
   if (uri->path != NULL)
 	ds_asprintf(&ds, "%s", percent_encode_chars(uri->path, "%;?&", 0));
   else
-	ds_asprintf(&ds, "");
+	ds_appendc(&ds, (int) '\0');
 
   for (i = 0; i < dsvec_len(uri->query_args); i++) {
 	a = (Uri_query_arg *) dsvec_ptr_index(uri->query_args, i);
@@ -1135,9 +1135,9 @@ http_set_user_agent_string(char *str)
 static int
 connect_to_server(Http *h, char *server, in_port_t port)
 {
-  int rc, sd;
-  struct sockaddr_in addr;
-  struct hostent *host;
+  int gai_code, rc, sd;
+  char *port_str;
+  struct addrinfo ai_hints, *ai;
 
   if (h->ssl_args != NULL) {
 	char *p;
@@ -1163,26 +1163,32 @@ connect_to_server(Http *h, char *server, in_port_t port)
 	return(0);
   }
 
-  addr.sin_family = AF_INET;
-  addr.sin_port = htons((u_short) port);
-  if ((host = gethostbyname(server)) == NULL) {
-	log_msg((LOG_ERROR_LEVEL, "gethostbyname failed for '%s'", server));
+  memset(&ai_hints, 0, sizeof(ai_hints));
+  ai_hints.ai_family = AF_INET;
+  ai_hints.ai_socktype = SOCK_STREAM;
+  ai_hints.ai_protocol = IPPROTO_TCP;
+  ai_hints.ai_flags = AI_CANONNAME;
+
+  ai = NULL;
+  port_str = ds_xprintf("%u", (unsigned int) port);
+  gai_code = getaddrinfo(server, port_str, &ai_hints, &ai);
+  if (gai_code != 0) {
+	log_msg((LOG_ERROR_LEVEL, "getaddrinfo failed for '%s'", server));
 	return(-1);
   }
 
-  addr.sin_addr = *(struct in_addr *) host->h_addr;
-
   if ((sd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
 	log_err((LOG_ERROR_LEVEL, "socket() failed"));
 	return(-1);
   }
 
-  if (connect(sd, (const struct sockaddr *) &addr, (socklen_t) sizeof(addr))
-	  == -1) {
+  if (connect(sd, ai->ai_addr, ai->ai_addrlen) == -1) {
 	log_err((LOG_ERROR_LEVEL, "connect() failed"));
 	close(sd);
+	freeaddrinfo(ai);
 	return(-1);
   }
+  freeaddrinfo(ai);
 
   log_msg((LOG_TRACE_LEVEL, "Connected to %s:%u", server, port));
 
@@ -1238,6 +1244,10 @@ write_to_server(Http *h, char *data, size_t length)
 		select(0, NULL, NULL, NULL, &tv);
 		continue;
 	  }
+	  if (errno == EPIPE && h->ssl_write_fd != -1) {
+		/* SSL process quit, probably.  A message ought to be superfluous... */
+		return(-1);
+	  }
 	  log_err((LOG_ERROR_LEVEL, "write_to_server() write error"));
 	  return(-1);
 	}
@@ -1278,12 +1288,14 @@ http_start_data(Http *h, Ds *body, size_t len)
 static int
 send_request(Http *h)
 {
-  int i, rc; 
+  int i, rc, sent_authorization; 
   char *p, *pre_path, *server;
   size_t body_len;
   in_port_t port;
   Ds *body, request;
 
+  sent_authorization = 0;
+
   if (h->uri->host == NULL || *h->uri->host == '\0')
 	server = "localhost";
   else
@@ -1340,11 +1352,78 @@ send_request(Http *h)
 	ds_asprintf(&request, "%s\r\n", p);
   }
 
+  /*
+   * If proxy authentication has been pre-selected, send it.
+   */
+  if (h->proxy_userinfo != NULL) {
+	char *basic_credentials;
+
+	mime_encode_base64((unsigned char *) h->proxy_userinfo,
+					   strlen(h->proxy_userinfo), &basic_credentials);
+	ds_asprintf(&request, "Proxy-Authorization: Basic %s\n", basic_credentials);
+	/* XXX the caller should zap h->proxy_userinfo */
+	log_msg((LOG_DEBUG_LEVEL, "Proxy-Authorization: Basic %s",
+			 basic_credentials));
+	strzap(basic_credentials);
+  }
+
+  /*
+   * Send the userinfo field as an Authorization header, but only if not
+   * already done.
+   * XXX HTTP Digest is not supported
+   */
+  if (h->userinfo != NULL) {
+	char *basic_credentials;
+
+	if (sent_authorization) {
+	  log_msg((LOG_ERROR_LEVEL, "Multiple Authorization headers requested"));
+	  return(-1);
+	}
+
+	mime_encode_base64((unsigned char *) h->userinfo, strlen(h->userinfo),
+					   &basic_credentials);
+	ds_asprintf(&request, "Authorization: Basic %s\n", basic_credentials);
+	/* XXX the caller should zap h->userinfo */
+	log_msg((LOG_DEBUG_LEVEL, "Authorization: Basic %s", basic_credentials));
+	log_msg((LOG_TRACE_LEVEL, "[Sent Authorization field from context]"));
+	strzap(basic_credentials);
+	sent_authorization++;
+  }
+
+  /*
+   * Send the userinfo field as an Authorization header, but only if not
+   * already done.
+   * XXX HTTP Digest is not supported
+   */
+  if (h->uri->userinfo != NULL) {
+	char *basic_credentials;
+
+	if (sent_authorization) {
+	  log_msg((LOG_ERROR_LEVEL, "Multiple Authorization headers requested"));
+	  return(-1);
+	}
+
+	mime_encode_base64((unsigned char *) h->uri->userinfo,
+					   strlen(h->uri->userinfo), &basic_credentials);
+	ds_asprintf(&request, "Authorization: Basic %s\n", basic_credentials);
+	/* XXX the caller should zap h->uri->userinfo */
+	log_msg((LOG_DEBUG_LEVEL, "Authorization: Basic %s",
+			 basic_credentials));
+	strzap(basic_credentials);
+	log_msg((LOG_TRACE_LEVEL, "[Sent userinfo field from URI]"));
+	sent_authorization++;
+  }
+
   if (h->cookie_args != NULL) {
 	if (use_authorization_header) {
 	  char *c, *encoded_auth;
 	  Ds ds;
 
+	  if (sent_authorization) {
+		log_msg((LOG_ERROR_LEVEL, "Multiple Authorization headers requested"));
+		return(-1);
+	  }
+
 	  /* Use an RFC 2617 style header rather than cookies. */
 	  ds_asprintf(&request, "Authorization: DACS ");
 	  ds_init(&ds);
@@ -1355,6 +1434,7 @@ send_request(Http *h)
 	  mime_encode_base64((unsigned char *) ds_buf(&ds), ds_len(&ds) - 1,
 						 &encoded_auth);
 	  ds_asprintf(&request, "%s\r\n", encoded_auth);
+	  sent_authorization++;
 	}
 	else {
 	  ds_asprintf(&request,
@@ -1412,8 +1492,8 @@ send_request(Http *h)
 	fprintf(stderr, "Request:\n%s", ds_buf(&request));
 
   rc = write_to_server(h, ds_buf(&request), ds_len(&request) - 1);
-
-  http_start_data(h, body, body_len);
+  if (rc != -1)
+	http_start_data(h, body, body_len);
 
   return(rc);
 }
@@ -1428,8 +1508,14 @@ http_open(Http *h, char *url)
 	  log_msg((LOG_ERROR_LEVEL, "SSL is not configured for https method"));
 	  rc = -1;
 	}
-	else
+	else {
 	  rc = send_request(h);
+	  /*
+	   * If producing debugging or tracing output, emit a blank line
+	   * to separate the request from whatever might be printed next.
+	   */
+	  log_msg((LOG_DEBUG_LEVEL, ""));
+	}
   }
   else {
 	log_msg((LOG_ERROR_LEVEL, "Error parsing URL: \"%s\"", url));
@@ -1608,6 +1694,8 @@ http_init(Http_version *version)
   h->cookie_args = NULL;
   h->in_data = 0;
   h->uri = NULL;
+  h->userinfo = NULL;
+  h->proxy_userinfo = NULL;
   h->method = HTTP_GET_METHOD;
   h->server_options = 0;
   h->response_headers = NULL;
@@ -1839,14 +1927,15 @@ http_get_response_body(Http *h, char **reply, int *reply_len)
  *
  * REPLY is set to point to the reply if the invocation is successful,
  * otherwise it points to an error message.  In either case the reply buffer
- * is obtained via malloc(3).
+ * is obtained via malloc(3) and is always null terminated.
+ *
  * NOTE:
  * REPLY_LEN is an input-output parameter; on input, it is the maximum length
  * of the reply to return (it doesn't apply to error messages, however); if
  * it is negative, the length of the reply will only be limited by available
  * resources.
  * As an output, REPLY_LEN is set to the actual length of the reply (again,
- * except for error messages).
+ * except for error messages), excluding the null terminator.
  * If successful, 0 is returned; -1 is returned if an error occurs.
  */
 int
@@ -1971,6 +2060,8 @@ http_post(char *url, Dsvec *post_args, char **cookies, char **reply,
 	  http_param(params, arg->name, arg->value, NULL, 0);
 	}
   }
+  else
+	params = NULL;
 
   st = http_invoke(url, HTTP_POST_METHOD, HTTP_SSL_URL_SCHEME,
 				   dsvec_len(params), (Http_params *) dsvec_base(params),
@@ -1989,18 +2080,18 @@ insert_file(Http_params *param)
 	Ds *ds;
 
 	if (feof(stdin)) {
-	  fprintf(stderr, "Can't read stdin more than once\n");
+	  fprintf(stderr, "Can't read stdin more than once.\n");
 	  return(-1);
 	}
 	if ((ds = ds_getf(stdin)) == NULL) {
-	  fprintf(stderr, "Could not read stdin\n");
+	  fprintf(stderr, "Could not read stdin.\n");
 	  return(-1);
 	}
 	buf = ds_buf(ds);
   }
   else {
 	if (load_file(param->filename, &buf, NULL) == -1) {
-	  fprintf(stderr, "Could not read file \"%s\"\n", param->filename);
+	  fprintf(stderr, "Could not read file \"%s\".\n", param->filename);
 	  return(-1);
 	}
   }
@@ -2038,7 +2129,7 @@ static void
 usage(void)
 {
 
-  fprintf(stderr, "Usage: http [flags ...] uri\n");
+  fprintf(stderr, "Usage: dacshttp [flags ...] uri\n");
   fprintf(stderr, "Flags:\n");
   fprintf(stderr, "  -delete            :\n");
   fprintf(stderr, "  -get               :\n");
@@ -2085,6 +2176,10 @@ usage(void)
   fprintf(stderr, "Obtain cookies to send from filename, one per line\n");
   fprintf(stderr, "  -ah                : ");
   fprintf(stderr, "Send any cookies using the Authorization header\n");
+  fprintf(stderr, "  -u | --user <username>:<password>: ");
+  fprintf(stderr, "Send an HTTP Basic auth Authorization header\n");
+  fprintf(stderr, "  -U | --proxyuser <username>:<password>: ");
+  fprintf(stderr, "Send an HTTP Basic auth Proxy-Authorization header\n");
   fprintf(stderr, "  -ssl command-line  : ");
   fprintf(stderr, "Connect using SSL via another program, e.g.,\n");
   fprintf(stderr, "-ssl \"/usr/local/sbin/stunnel -r vs:443 -c");
@@ -2214,7 +2309,7 @@ dacshttp_main(int argc, char **argv, int do_init, void *main_out)
 	  }
 	  if (http_version->major > HTTP_VERSION_MAJOR
 		  || http_version->minor > HTTP_VERSION_MINOR) {
-		fprintf(stderr, "Unimplemented HTTP version\n");
+		fprintf(stderr, "Unimplemented HTTP version.\n");
 		goto fail;
 	  }
 	  continue;
@@ -2289,6 +2384,24 @@ dacshttp_main(int argc, char **argv, int do_init, void *main_out)
 	  continue;
 	}
 
+	if (streq(argv[i], "-u") || streq(argv[i], "--user")) {
+	  if (argv[++i] == NULL)
+		goto fail;
+	  if (h->userinfo != NULL)
+		goto fail;
+	  h->userinfo = strdup(argv[i]);
+	  continue;
+	}
+
+	if (streq(argv[i], "-U") || streq(argv[i], "--proxyuser")) {
+	  if (argv[++i] == NULL)
+		goto fail;
+	  if (h->proxy_userinfo != NULL)
+		goto fail;
+	  h->proxy_userinfo = strdup(argv[i]);
+	  continue;
+	}
+
 	if (strcaseeq(argv[i], "-user-agent")) {
 	  if (argv[++i] == NULL)
 		goto fail;
@@ -2340,7 +2453,7 @@ dacshttp_main(int argc, char **argv, int do_init, void *main_out)
 	  if (argv[++i] == NULL)
 		goto fail;
 	  if ((fp = fopen(argv[i], "r")) == NULL) {
-		fprintf(stderr, "Can't read headers from \"%s\"\n", argv[i]);
+		fprintf(stderr, "Can't read headers from \"%s\".\n", argv[i]);
 		goto fail;
 	  }
 
@@ -2371,7 +2484,7 @@ dacshttp_main(int argc, char **argv, int do_init, void *main_out)
 		fp = stdin;
 	  else {
 		if ((fp = fopen(argv[i], "r")) == NULL) {
-		  fprintf(stderr, "Can't read body from \"%s\"\n", argv[i]);
+		  fprintf(stderr, "Can't read body from \"%s\".\n", argv[i]);
 		  goto fail;
 		}
 	  }
@@ -2379,7 +2492,7 @@ dacshttp_main(int argc, char **argv, int do_init, void *main_out)
 	  dsio_set(h->message_body, fp, NULL, 0, 0);
 	  if (dsio_load(h->message_body) == -1) {
 		fclose(fp);
-		fprintf(stderr, "Error reading body from \"%s\"\n", argv[i]);
+		fprintf(stderr, "Error reading body from \"%s\".\n", argv[i]);
 		goto fail;
 	  }
 	  fclose(fp);
@@ -2389,7 +2502,7 @@ dacshttp_main(int argc, char **argv, int do_init, void *main_out)
 
 	if (strcaseeq(argv[i], "-cookies")) {
 	  if (set_cookies_from_file(h, argv[i + 1]) == -1) {
-		fprintf(stderr, "Couldn't get cookies from \"%s\"\n", argv[i + 1]);
+		fprintf(stderr, "Couldn't get cookies from \"%s\".\n", argv[i + 1]);
 		return(-1);
 	  }
 	  i++;
@@ -2431,6 +2544,9 @@ dacshttp_main(int argc, char **argv, int do_init, void *main_out)
   if ((request_uri = argv[i]) == NULL)
 	goto fail;
 
+  if (argv[i + 1] != NULL)
+	goto fail;
+
   if (verbose_flag == 1)
 	log_set_level(ld, LOG_DEBUG_LEVEL);
   else if (verbose_flag > 1)
@@ -2439,7 +2555,7 @@ dacshttp_main(int argc, char **argv, int do_init, void *main_out)
  http_invoke:
 
   if ((uri = http_parse_uri(request_uri)) == NULL) {
-	fprintf(stderr, "Parse of URL '%s' failed\n", request_uri);
+	fprintf(stderr, "Parse of URL '%s' failed.\n", request_uri);
 	return(-1);
   }
 
@@ -2499,7 +2615,6 @@ dacshttp_main(int argc, char **argv, int do_init, void *main_out)
 	  fprintf(stderr, "Query Arg: %s=\"%s\"\n", r->name, r->value);
 	}
 	fprintf(stderr, "User-Agent: %s\n", user_agent_string);
-	fprintf(stderr, "\n");
   }
 
   http_set_post_params(h, dsvec_len(post_params),
@@ -2631,7 +2746,7 @@ dacshttp_main(int argc, char **argv, int do_init, void *main_out)
 	}
 	else {
 	  if (nredirects++ >= HTTP_MAX_REDIRECTS) {
-		fprintf(stderr, "Maximum number of redirects (%d) has been exceeded\n",
+		fprintf(stderr, "Maximum number of redirects (%d) has been exceeded.\n",
 				HTTP_MAX_REDIRECTS);
 		goto done;
 	  }
