@@ -64,7 +64,7 @@ static const char copyright[] =
 "Copyright (c) 2003-2012\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: local_ldap_auth.c 2577 2012-03-07 20:53:07Z brachman $";
+  "$Id: local_ldap_auth.c 2755 2014-12-31 17:57:46Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -386,7 +386,7 @@ do_init(char *host, int port)
 
   if (ldap_set_option(ld, LDAP_OPT_REFERRALS, LDAP_OPT_OFF)
 	  != LDAP_OPT_SUCCESS) {
-	log_msg((LOG_ERROR_LEVEL, "Could not set LDAP_OPT_REFERRALS off"));
+	log_msg((LOG_ERROR_LEVEL, "Cannot set LDAP_OPT_REFERRALS off"));
 	return(NULL);
   }
 
@@ -394,14 +394,14 @@ do_init(char *host, int port)
   protocol = LDAP_VERSION3;
   if (ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION, &protocol)
 	  != LDAP_OPT_SUCCESS) {
-	log_msg((LOG_ERROR_LEVEL, "Could not set LDAP_OPT_PROTOCOL_VERSION %d",
+	log_msg((LOG_ERROR_LEVEL, "Cannot set LDAP_OPT_PROTOCOL_VERSION %d",
 			 protocol));
 	return(NULL);
   }
 
   debug = (trace_level > 1);
   if (ldap_set_option(NULL, LDAP_OPT_DEBUG_LEVEL, &debug) != LDAP_OPT_SUCCESS)
-	log_msg((LOG_DEBUG_LEVEL, "Could not set LDAP_OPT_DEBUG_LEVEL %d", debug));
+	log_msg((LOG_DEBUG_LEVEL, "Cannot set LDAP_OPT_DEBUG_LEVEL %d", debug));
 
   return(ld);
 }
@@ -445,7 +445,7 @@ local_ldap_auth_by_dn(Uri *url, char *username, char *password,
 	n = fetch_entry(ld, user_dn, scope, NULL, NULL,
 					attrs, attrsonly, NULL, NULL, timeout, -1, NULL, avas);
 	if (n == -1) {
-	  log_msg((LOG_ERROR_LEVEL, "Can't read entry for DN %s", user_dn));
+	  log_msg((LOG_ERROR_LEVEL, "Cannot read entry for DN %s", user_dn));
 	  ldap_unbind(ld);
 	  return(-1);
 	}
@@ -674,8 +674,18 @@ main(int argc, char **argv)
 	goto fail;
   }
 
+  /*
+   * If called as a roles module, the username should be passed as
+   * DACS_USERNAME (an authenticated name) but if called as an authentication
+   * module, the username should be passed as USERNAME (an unauthenticated
+   * name).  It is an error if both parameters are present.
+   */
   for (i = 0; i < kwv->nused; i++) {
-	if (streq(kwv->pairs[i]->name, "USERNAME") && username == NULL)
+	if (!local_ldap_roles
+		&& streq(kwv->pairs[i]->name, "USERNAME") && username == NULL)
+	  username = kwv->pairs[i]->val;
+	else if (local_ldap_roles
+			 && streq(kwv->pairs[i]->name, "DACS_USERNAME") && username == NULL)
 	  username = kwv->pairs[i]->val;
 	else if (streq(kwv->pairs[i]->name, "PASSWORD") && password == NULL)
 	  password = kwv->pairs[i]->val;
@@ -761,7 +771,7 @@ main(int argc, char **argv)
 	  auth_ldap_username_url = p;
 
 	if ((url = uri_parse(auth_ldap_username_url)) == NULL) {
-	  errmsg = ds_xprintf("Can't parse LDAP_USERNAME_URL: \"%s\"",
+	  errmsg = ds_xprintf("Cannot parse LDAP_USERNAME_URL: \"%s\"",
 						  auth_ldap_username_url);
 	  goto fail;
 	}
@@ -838,7 +848,7 @@ main(int argc, char **argv)
 	  ns_args[2].kwv = NULL;
 	  acs_expr_string(p_e, ns_args, &auth_ldap_search_filter);
 	  if (auth_ldap_search_filter == NULL) {
-		errmsg = "Could not form search filter";
+		errmsg = "Cannot form search filter";
 		goto fail;
 	  }
 	}
@@ -846,7 +856,8 @@ main(int argc, char **argv)
 	  auth_ldap_search_filter = p;
 
 	if ((url = uri_parse(auth_ldap_admin_url)) == NULL) {
-	  errmsg = "Can't parse LDAP_ADMIN_URL";
+	  errmsg = ds_xprintf("Cannot parse LDAP_ADMIN_URL: \"%s\"",
+						  auth_ldap_admin_url);
 	  goto fail;
 	}
 	if (url->scheme != NULL && strcaseeq(url->scheme, "ldap")) {
@@ -886,10 +897,10 @@ main(int argc, char **argv)
 
   if (!success) {
 	if (username != NULL)
-	  errmsg = ds_xprintf("Could not bind to directory as user \"%s\"",
+	  errmsg = ds_xprintf("Cannot bind to directory as user \"%s\"",
 						  username);
 	else
-	  errmsg = "Could not bind to directory";
+	  errmsg = "Cannot bind to directory";
 	goto fail;
   }
 
