@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2015
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -85,16 +85,16 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2015\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: loglib.c 2594 2012-10-19 17:28:49Z brachman $";
+  "$Id: loglib.c 2796 2015-04-24 20:15:47Z brachman $";
 #endif
 
-#include <syslog.h>
-
 #include "log.h"
 
+#include <syslog.h>
+
 Log_desc *log_current_logd = NULL;
 
 typedef struct Level_map {
@@ -103,7 +103,7 @@ typedef struct Level_map {
   Log_level level;
 } Level_map;
 
-/* Indexed by Log_level */
+/* Indexed by Log_level masked by LOG_LEVEL_MASK. */
 static Level_map log_levels[] = {
   { "trace",    "tracing",          LOG_TRACE_LEVEL },
   { "debug",    "debugging",        LOG_DEBUG_LEVEL },
@@ -158,9 +158,14 @@ log_lookup_level(char *level_name)
 {
   int i;
 
+  /* An unfortunate special case. */
+  if (strcaseeq(level_name, "startup"))
+	return(LOG_STARTUP_LEVEL);
+
   for (i = 0; log_levels[i].name != NULL; i++) {
 	if (strcaseeq(level_name, log_levels[i].name))
 	  return(log_levels[i].level);
+
 	if (log_levels[i].alt_names != NULL) {
 	  int j;
 	  Dsvec *dsv;
@@ -182,6 +187,10 @@ log_level_to_name(Log_level level)
 {
   int i;
 
+  /* An unfortunate special case. */
+  if (level == LOG_STARTUP_LEVEL)
+	return("startup");
+
   for (i = 0; log_levels[i].name != NULL; i++) {
 	if (log_levels[i].level == level)
 	  return(log_levels[i].name);
@@ -195,7 +204,8 @@ log_is_valid_level(Log_level level)
 {
   int lev;
 
-  if (level == LOG_NONE_LEVEL || level == LOG_FORCE_LEVEL)
+  if (level == LOG_NONE_LEVEL || level == LOG_FORCE_LEVEL
+		|| level == LOG_STARTUP_LEVEL)
 	return(1);
 
   lev = level & LOG_LEVEL_MASK;
@@ -907,6 +917,19 @@ log_filter(Log_desc *logd, Log_filter **lf_match)
   if (logd->event.level == LOG_FORCE_LEVEL)
 	return(1);
 
+  /*
+   * If we are suppressing startup messages (which should only be done
+   * if you are certain it is necessary), do not log this message; otherwise,
+   * this is equivalent to a forced message, which must be emitted.
+   */
+  if (logd->event.level == LOG_STARTUP_LEVEL) {
+#ifdef ENABLE_HUSH_STARTUP_LOGGING
+	return(0);
+#else
+	return(1);
+#endif
+  }
+
   event_lev = log_get_level(logd->event.level);
 
   if (event_lev < LOG_LOWEST_LEVEL || event_lev > LOG_HIGHEST_LEVEL)
