@@ -1,15 +1,15 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2016\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: misc.c 2594 2012-10-19 17:28:49Z brachman $";
+  "$Id: misc.c 2878 2016-05-16 23:02:19Z brachman $";
 #endif
 
 #ifdef DSSLIB
@@ -22,6 +22,96 @@ static const char revid[] =
 
 static const char *log_module_name = "misc";
 
+static int
+abs_mode(char *mode_str, mode_t *mode)
+{
+
+  return(strnum_b(mode_str, STRNUM_MODE_T, 8, mode, NULL));
+}
+
+/*
+ * Create PATHNAME with MODE_STR if it does not already exist, otherwise set the
+ * access and modification times of PATHNAME to the current time (and not change its
+ * mode).
+ */
+int
+touch_file(char *pathname, char *mode_str, char **errmsg)
+{
+  int fd, st;
+  mode_t mode;
+  struct stat statb;
+
+  st = stat(pathname, &statb);
+  if (st == -1) {
+    if (errno != ENOENT) {
+	  *errmsg = strdup(strerror(errno));
+      return(-1);
+	}
+
+    /* Doesn't exist, create it. */
+    if (mode_str == NULL)
+      mode = 0644;
+    else if (abs_mode(mode_str, &mode) == -1) {
+      *errmsg = "Invalid mode";
+      return(-1);
+    }
+
+    fd = open(pathname, O_WRONLY | O_CREAT, mode);
+    if (fd == -1 || fstat(fd, &statb) != 0 || close(fd) != 0) {
+      *errmsg = strdup(strerror(errno));
+      return(-1);
+    }
+  }
+  else {
+    /* Exists, update it. */
+    if (utimes(pathname, NULL) == -1) {
+      *errmsg = strdup(strerror(errno));
+      return(-1);
+    }
+  }
+
+  return(0);
+}
+
+int
+chmod_file(char *pathname, char *mode_str, char **errmsg)
+{
+  mode_t mode;
+
+  if (abs_mode(mode_str, &mode) != -1) {
+	if (chmod(pathname, mode) == -1) {
+	  *errmsg = strdup(strerror(errno));
+	  return(-1);
+	}
+  }
+  else {
+	*errmsg = "Invalid mode";
+	return(-1);
+  }
+
+  return(1);
+}
+
+int
+mkdir_file(char *dirname, char *mode_str, char **errmsg)
+{
+  mode_t mode;
+
+  if (mode_str == NULL)
+	mode = 0755;
+  else if (abs_mode(mode_str, &mode) == -1) {
+	*errmsg = "Invalid mode";
+	return(-1);
+  }
+
+  if (mkdir(dirname, mode) == -1) {
+	*errmsg = strdup(strerror(errno));
+	return(-1);
+  }
+
+  return(1);
+}
+
 /*
  * Allocate a buffer to hold FILENAME and read the contents into it.
  * If FILENAME is NULL, read stdin.
@@ -123,8 +213,8 @@ filterthru(char **argv, char **env, int *read_fd, int *write_fd,
   log_msg((LOG_TRACE_LEVEL, "filterthru: executing %s", argv[0]));
   if (log_test(LOG_TRACE_LEVEL)) {
 	log_msg((LOG_TRACE_LEVEL, "with arguments:"));
-	for (i = 1; argv[i] != NULL; i++)
-	  log_msg((LOG_TRACE_LEVEL, "  \"%s\"", argv[i]));
+	for (i = 0; argv[i] != NULL; i++)
+	  log_msg((LOG_TRACE_LEVEL, "argv[%d]: \"%s\"", i, argv[i]));
   }
 
   if ((pid = fork()) == 0) {
@@ -176,6 +266,7 @@ filterthru(char **argv, char **env, int *read_fd, int *write_fd,
   if (pidp != NULL)
 	*pidp = pid;
 
+  log_msg((LOG_TRACE_LEVEL, "filterthru: parent continuing"));
   return(0);
 
  err:
@@ -270,8 +361,12 @@ log_exec_msg(int level, char *fmt, ...)
   va_list ap;
   Ds ds;
   FILE *fp;
+  extern Log_level current_log_level;
 
-  if ((fp = log_stream) == NULL)
+  if (level < current_log_level)
+	return(0);
+
+ if ((fp = log_stream) == NULL)
 	fp = stderr;
 
   va_start(ap, fmt);
@@ -299,7 +394,7 @@ log_exec_msg(int level, char *fmt, ...)
 char *
 make_rfc822_date_string(struct tm *tm, int yyyy, int use_gmt)
 {
-  int st;
+  size_t st;
   char *buf;
 
   if (tm == NULL) {
@@ -345,7 +440,7 @@ make_rfc822_date_string(struct tm *tm, int yyyy, int use_gmt)
 char *
 make_utc_date_string(struct tm *tm)
 {
-  int st;
+  size_t st;
   char *buf;
 
   if (tm == NULL) {
@@ -369,7 +464,7 @@ make_utc_date_string(struct tm *tm)
 char *
 make_local_date_string(struct tm *tm, int show_zone)
 {
-  int st;
+  size_t st;
   char *buf;
 
   if (tm == NULL) {
@@ -467,7 +562,7 @@ tm_get_gmtoff(struct tm *xtm)
 char *
 make_short_local_date_string(struct tm *tm)
 {
-  int st;
+  size_t st;
   char *buf;
 
   if (tm == NULL) {
@@ -510,8 +605,8 @@ make_datetime(struct tm *tm)
 				   tm->tm_mon + 1,
 				   tm->tm_mday,
 				   tm->tm_hour, tm->tm_min, tm->tm_sec,
-				   tm_get_gmtoff(tm) / 3600,
-				   tm_get_gmtoff(tm) % 3600);
+				   (int) tm_get_gmtoff(tm) / 3600,
+				   (int) tm_get_gmtoff(tm) % 3600);
 
   return(str);
 }
@@ -882,4 +977,3 @@ escape_xml_attribute(char *str, int quote_char)
   ds_appendc(&ds, (int) '\0');
   return(ds_buf(&ds));
 }
-
