@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -43,10 +43,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2016\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: mkkey.c 2586 2012-03-15 16:21:40Z brachman $";
+  "$Id: mkkey.c 2912 2016-10-18 19:54:07Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -127,7 +127,6 @@ mk_rsa(int rsa_key_bits, char **public_key, char **private_key)
 #if (OPENSSL_VERSION_NUMBER >= 0x0090800fL)
   int i;
   BIGNUM *e;
-  BN_GENCB cb;
 
   rsa = RSA_new();
   e = BN_new();
@@ -136,8 +135,15 @@ mk_rsa(int rsa_key_bits, char **public_key, char **private_key)
 	  BN_set_bit(e, i);
   }
 
-  BN_GENCB_set_old(&cb, NULL, NULL);
-  st = RSA_generate_key_ex(rsa, rsa_key_bits, e, &cb);
+#ifdef OLD
+  {
+	BN_GENCB cb;
+	BN_GENCB_set_old(&cb, NULL, NULL);
+	st = RSA_generate_key_ex(rsa, rsa_key_bits, e, &cb);
+  }
+#else
+  st = RSA_generate_key_ex(rsa, rsa_key_bits, e, NULL);
+#endif
   BN_free(e);
 #else
   rsa = RSA_generate_key(rsa_key_bits, e_value, NULL, NULL);
@@ -210,10 +216,10 @@ crypt_keys_create(unsigned char *auth_key, unsigned char *hmac_key,
   ck->hmac_key_len = CRYPTO_HMAC_KEY_LENGTH;
 
   /* Convert keys to their external (textual) form. */
-  len = strba64(ck->auth_key, AUTH_CRYPT_KEY_LENGTH, &ck->auth_key_str);
+  len = strba64(ck->auth_key, AUTH_CRYPT_KEY_LENGTH, &ck->auth_key_str) + 1;
   memzap(auth_key, AUTH_CRYPT_KEY_LENGTH);
 
-  len = strba64(ck->hmac_key, CRYPTO_HMAC_KEY_LENGTH, &ck->hmac_key_str);
+  len = strba64(ck->hmac_key, CRYPTO_HMAC_KEY_LENGTH, &ck->hmac_key_str) + 1;
   memzap(hmac_key, CRYPTO_HMAC_KEY_LENGTH);
 
   if (mk_rsa(rsa_bits, &ck->public_key_str, &ck->private_key_str) == -1)
@@ -251,6 +257,7 @@ crypt_keys_create(unsigned char *auth_key, unsigned char *hmac_key,
   return(ck);
 }
 
+#ifdef NOTDEF
 Crypt_keys *
 crypt_keys_from_passwd(char *passwd)
 {
@@ -260,9 +267,9 @@ crypt_keys_from_passwd(char *passwd)
 
   rsa_bits = CRYPT_KEYS_RSA_KEY_LENGTH_DEFAULT;
 
-  derived_passwd = crypto_pbkdf2((unsigned char *) passwd, -1, NULL, 0, 1024,
-								 AUTH_CRYPT_KEY_LENGTH
-								 + CRYPTO_HMAC_KEY_LENGTH);
+  derived_passwd = crypto_pbkdf2((unsigned char *) passwd, -1, NULL, 0,
+								 "SHA1", 1024,
+								 AUTH_CRYPT_KEY_LENGTH + CRYPTO_HMAC_KEY_LENGTH);
   auth_key_bytes = derived_passwd;
   hmac_key_bytes = auth_key_bytes + AUTH_CRYPT_KEY_LENGTH;
 
@@ -284,9 +291,9 @@ crypt_encrypt_buf_to_file(FILE *fp, Crypt_keys *ck, unsigned char *plaintext,
   enc_len = crypto_encrypt_string(ck, plaintext, plen, &encrypted);
 
   if (write_text == 85)
-	len = strba85(encrypted, enc_len, &out_buf) - 1;
+	len = strba85(encrypted, enc_len, &out_buf);
   else if (write_text == 64)
-	len = strba64(encrypted, enc_len, &out_buf) - 1;
+	len = strba64(encrypted, enc_len, &out_buf);
   else {
 	len = enc_len;
 	out_buf = (char *) encrypted;
@@ -299,14 +306,12 @@ crypt_encrypt_buf_to_file(FILE *fp, Crypt_keys *ck, unsigned char *plaintext,
   return(0);
 }
 
-#ifdef NOTDEF
 int
 crypt_encrypt_stream(FILE *fp_in, FILE *fp_out, Crypt_keys *ck,
 					 int write_text)
 {
 
 }
-#endif
 
 int
 crypt_decrypt_file_to_buf(FILE *fp, Crypt_keys *ck, unsigned char **plaintext,
@@ -328,7 +333,6 @@ crypt_decrypt_file_to_buf(FILE *fp, Crypt_keys *ck, unsigned char **plaintext,
   return(st);
 }
 
-#ifdef NOTDEF
 int
 crypt_decrypt_stream(FILE *fp_in, FILE *fp_out, Crypt_keys *ck,
 					 int write_text)
@@ -757,7 +761,7 @@ return(0);
 	}
 
 	if (do_check) {
-	  if (!quiet_flag && ck->fed_id != NULL)
+	  if (!dacs_quiet_flag && ck->fed_id != NULL)
 		fprintf(stderr, "OK fed_id=\"%s\"\n", ck->fed_id);
 
 	  exit(0);
@@ -869,9 +873,9 @@ return(0);
 
 #ifdef NOTDEF
   /* Convert to the external form and print. */
-  len = strba64(auth_key, AUTH_CRYPT_KEY_LENGTH, &auth_keystr);
+  len = strba64(auth_key, AUTH_CRYPT_KEY_LENGTH, &auth_keystr) + 1;
   memzap(auth_key, AUTH_CRYPT_KEY_LENGTH);
-  len = strba64(hmac_key, CRYPTO_HMAC_KEY_LENGTH, &hmac_keystr);
+  len = strba64(hmac_key, CRYPTO_HMAC_KEY_LENGTH, &hmac_keystr) + 1;
   memzap(hmac_key, CRYPTO_HMAC_KEY_LENGTH);
 
   if (mk_rsa(rsa_bits, &public_key, &private_key) == -1)
@@ -933,7 +937,7 @@ return(0);
    * figure out how CGI (non-interactive) programs would also be able to access
    * the key file.
    */
-  if (!quiet_flag)
+  if (!dacs_quiet_flag)
 	fprintf(stderr, "Please check ownership and mode of '%s'\n", keyfile);
 
   return(0);
