@@ -50,7 +50,7 @@ static const char copyright[] =
 "Copyright (c) 2003-2012\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: notices.c 2586 2012-03-15 16:21:40Z brachman $";
+  "$Id: notices.c 2610 2012-10-19 18:02:36Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -186,12 +186,13 @@ parse_nat_cookie(char *cookie)
   for (v = kwv_iter_first(iter); v != NULL ; v = kwv_iter_next(iter)) {
 	if (strcaseeq(v->name, "ResourceURIs")) {
 	  nat->resource_uris = strdup(v->val);
-	  if ((nat->resources = strsplit_re(v->val, " [ ]*", 0, &errmsg)) == NULL)
+	  if ((nat->resources = strsplit_re(v->val, " [ ]*", 0, 0, &errmsg))
+		  == NULL)
 		goto fail;
 	}
 	else if (strcaseeq(v->name, "NoticeURIs")) {
 	  nat->notice_uris = strdup(v->val);
-	  if ((nat->notices = strsplit_re(v->val, " [ ]*", 0, &errmsg)) == NULL)
+	  if ((nat->notices = strsplit_re(v->val, " [ ]*", 0, 0, &errmsg)) == NULL)
 		goto fail;
 	}
   }
@@ -978,7 +979,8 @@ main(int argc, char **argv)
 	}
 
 	if (*notice_uris == '\0'
-		|| (notices = strsplit_re(notice_uris, " [ ]*", 0, &errmsg)) == NULL) {
+		|| (notices = strsplit_re(notice_uris, " [ ]*", 0, 0, &errmsg))
+		== NULL) {
 	  errmsg = "Invalid NOTICE_URIS argument";
 	  goto fail;
 	}
