@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2013
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -43,12 +43,14 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2013\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: pamlib.c 2594 2012-10-19 17:28:49Z brachman $";
+  "$Id: pamlib.c 2669 2013-10-22 17:03:44Z brachman $";
 #endif
 
+#include "auth.h"
+
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/socket.h>
@@ -57,9 +59,6 @@ static const char revid[] =
 #include <netinet/tcp.h>
 #include <arpa/inet.h>
 
-#include "auth.h"
-
-
 static const char *log_module_name = "pamlib";
 
 /*
@@ -598,20 +597,20 @@ make_json_prompts(Auth_prompts *prompts, int html, char **json)
   Ds ds;
 
   ds_init(&ds);
-  ds_asprintf(&ds, "{ prompts:");
-  ds_asprintf(&ds, " { transid:\"%s\"", prompts->transid);
+  ds_asprintf(&ds, "{ \"prompts\":");
+  ds_asprintf(&ds, " { \"transid\":\"%s\"", prompts->transid);
   if (prompts->head != NULL)
-	ds_asprintf(&ds, ",\n prompt: [ ");
+	ds_asprintf(&ds, ",\n \"prompt\": [ ");
 
   for (pr = prompts->head; pr != NULL; pr = pr->next) {
 	if (pr != prompts->head)
 	  ds_asprintf(&ds, ",\n");
 
-	ds_asprintf(&ds, "{ type:\"%s\"", pr->type);
+	ds_asprintf(&ds, "{ \"type\":\"%s\"", pr->type);
 	if (pr->label != NULL)
-	  ds_asprintf(&ds, ", label:\"%s\"", pr->label);
+	  ds_asprintf(&ds, ", \"label\":\"%s\"", pr->label);
 	if (pr->varname != NULL)
-	  ds_asprintf(&ds, ", varname:\"%s\"", pr->varname);
+	  ds_asprintf(&ds, ", \"varname\":\"%s\"", pr->varname);
 	ds_asprintf(&ds, " }\n");
   }
 if (prompts->head != NULL)
