@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2015
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -147,10 +147,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2015\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: register.c 2542 2012-01-11 19:40:13Z brachman $";
+  "$Id: register.c 2844 2015-08-24 22:50:16Z brachman $";
 #endif
 
 #include "dacs_api.h"
@@ -343,7 +343,7 @@ make_secure_link(Crypt_keys *ck, char *link_args)
  * Return 0 if successful, -1 otherwise.
  */
 static int
-register_create(Vfs_handle *h, Passwd_digest_alg alg, Register_args *args)
+register_create(Vfs_handle *h, Digest_desc *dd, Register_args *args)
 {
   int st;
   char *new_account, *new_password;
@@ -352,7 +352,7 @@ register_create(Vfs_handle *h, Passwd_digest_alg alg, Register_args *args)
   new_password = args->new_password;
 
   log_msg((LOG_DEBUG_LEVEL, "Creating new account for \"%s\"", new_account));
-  st = pw_add_entry(h, new_account, new_password, alg, PW_ENABLED, NULL);
+  st = pw_add_entry(h, new_account, new_password, dd, PW_STATE_ENABLED, NULL);
 
   return(st);
 }
@@ -365,7 +365,7 @@ register_create(Vfs_handle *h, Passwd_digest_alg alg, Register_args *args)
  * Return 0 if successful, -1 otherwise.
  */
 static int
-register_pwreset(Vfs_handle *h, Passwd_digest_alg alg, Register_args *args)
+register_pwreset(Vfs_handle *h, Digest_desc *dd, Register_args *args)
 {
   int st;
   char *account, *new_password;
@@ -374,7 +374,7 @@ register_pwreset(Vfs_handle *h, Passwd_digest_alg alg, Register_args *args)
   new_password = args->new_password;
 
   log_msg((LOG_DEBUG_LEVEL, "Password reset for account \"%s\"", account));
-  st = pw_reset_entry(h, account, new_password, alg, PW_UNCHANGED,
+  st = pw_reset_entry(h, account, new_password, dd, PW_STATE_UNCHANGED,
 					  PW_OP_RESET, NULL);
 
   return(st);
@@ -389,7 +389,7 @@ register_pwreset(Vfs_handle *h, Passwd_digest_alg alg, Register_args *args)
  * Return 0 if successful, -1 otherwise.
  */
 static int
-register_rename(Vfs_handle *h, Passwd_digest_alg alg, Register_args *args)
+register_rename(Vfs_handle *h, Digest_desc *dd, Register_args *args)
 {
   int st;
   char *old_account, *password, *new_account;
@@ -517,18 +517,19 @@ dacsregister_main(int argc, char **argv, int do_init)
   int admin, i, xargc, n, need_codeword, st, use_question;
   unsigned int link_lifetime_secs, ncookies;
   char *arg_str, *link_args, *op_name, *op_str, *redirect_url, *to_addr;
-  char *codeword_arg, *digest_name, *expires, *item_type, *codeword;
+  char *codeword_arg, *expires, *item_type, *codeword;
   char **xargv, *errmsg, *p, *remote_addr, *seqno_str, *ustamp;
   char *codeword_template, *error_handler, *email_success_handler;
   char *codeword_handler, *email_template, *success_handler;
+  const char *digest_name;
   time_t now;
   Cookie *cookies;
   Credentials *cr, *credentials, *selected;
   Crypt_keys *ck;
+  Digest_desc *dd;
   Html_header_conf *hc;
   Kwv *kwv;
   Nickname_conf nicknames;
-  Passwd_digest_alg alg;
   Proc_lock *lock;
   Register_args *args, *new_args;
   Register_phase next_phase;
@@ -544,6 +545,8 @@ dacsregister_main(int argc, char **argv, int do_init)
   ustamp = NULL;
   error_handler = NULL;
 
+  args = new_args = NULL;
+
   xargc = argc;
   xargv = argv;
   if (dacs_init(DACS_WEB_SERVICE, &argc, &argv, &kwv, &errmsg) == -1) {
@@ -616,7 +619,7 @@ dacsregister_main(int argc, char **argv, int do_init)
   }
 
   digest_name = NULL;
-  if (passwd_get_digest_algorithm(&digest_name, &alg) == -1) {
+  if ((dd = passwd_get_digest_algorithm(digest_name)) == NULL) {
 	errmsg = "Configuration error";
 	goto fail;
   }
@@ -796,11 +799,11 @@ dacsregister_main(int argc, char **argv, int do_init)
 	  }
 
 	  if (args->op == OP_CREATE)
-		st = register_create(h, alg, args);
+		st = register_create(h, dd, args);
 	  else if (args->op == OP_PWRESET)
-		st = register_pwreset(h, alg, args);
+		st = register_pwreset(h, dd, args);
 	  else if (args->op == OP_RENAME)
-		st = register_rename(h, alg, args);
+		st = register_rename(h, dd, args);
 	  else
 		st = -1;
 
@@ -995,7 +998,7 @@ dacsregister_main(int argc, char **argv, int do_init)
 		errmsg = "Missing PASSWORD argument";
 		goto fail;
 	  }
-	  if (pw_check_passwd(h, args->account, args->password, alg) != 0) {
+	  if (pw_check_passwd(h, args->account, args->password, dd) != 0) {
 		errmsg = "Permission denied";
 		goto fail;
 	  }
