@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2014
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -46,18 +46,16 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2014\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: select_credentials.c 2594 2012-10-19 17:28:49Z brachman $";
+  "$Id: select_credentials.c 2710 2014-09-04 23:10:39Z brachman $";
 #endif
 
 #include "dacs.h"
 
 static const char *log_module_name = "dacs_select_credentials";
 
-static int emit_xml = 0;
-
 int
 main(int argc, char **argv)
 {
@@ -65,6 +63,7 @@ main(int argc, char **argv)
   char *cookie_syntax, *errmsg, *remote_addr, **xargv;
   char *jurisdiction, *operation, *username;
   unsigned int ncookies;
+  Common_status common_status;
   Cookie *cookies;
   Credentials *cr, *credentials;
   Html_header_conf *hc;
@@ -81,9 +80,7 @@ main(int argc, char **argv)
   dacs_init_allow_dups_default = 1;
   if (dacs_init(DACS_WEB_SERVICE, &argc, &argv, &kwv, &errmsg) == -1) {
   fail:
-	if (emit_xml) {
-	  Common_status common_status;
-
+	if (test_emit_xml_format()) {
 	  emit_xml_header(stdout, "dacs_select_credentials");
 	  printf("<%s>\n", make_xml_root_element("dacs_select_credentials"));
 	  init_common_status(&common_status, NULL, NULL, errmsg);
@@ -91,6 +88,12 @@ main(int argc, char **argv)
 	  printf("</dacs_select_credentials>\n");
 	  emit_xml_trailer(stdout);
 	}
+	else if (test_emit_format(EMIT_FORMAT_JSON)) {
+	  emit_json_header(stdout, "dacs_select_credentials");
+      init_common_status(&common_status, NULL, NULL, errmsg);
+      fprintf(stdout, "%s", make_json_common_status(&common_status));
+	  emit_json_trailer(stdout);
+	}
 	else {
 	  hc->title = ds_xprintf("DACS Selected Credentials for Federation %s",
 							 non_null(conf_val(CONF_FEDERATION_NAME)));
@@ -136,8 +139,7 @@ main(int argc, char **argv)
 	goto fail;
   }
 
-  log_msg((LOG_TRACE_LEVEL, "There are %d cookies, %d credentials",
-		   ncookies, ncred));
+  log_msg((LOG_TRACE_LEVEL, "Cookies=%d, Credentials=%d", ncookies, ncred));
 
   if ((k = kwv_lookup(kwv, "OPERATION")) == NULL) {
 	errmsg = "No OPERATION argument provided";
@@ -239,15 +241,19 @@ main(int argc, char **argv)
 	  }
 	}
 
-	if (ndeselected == nselected) {
-	  make_set_scredentials_cookie_header(NULL, &str);
-	  select_unauth = 1;
+	if (ndeselected) {
+	  if (ndeselected == nselected) {
+		make_set_scredentials_cookie_header(NULL, &str);
+		select_unauth = 1;
+	  }
+	  else
+		make_set_scredentials_cookie_header(credentials, &str);
 	}
-	else if (ndeselected)
-	  make_set_scredentials_cookie_header(credentials, &str);
 	else {
 	  /* No argument matched... */
-	  str = "\n";
+	  log_msg((LOG_TRACE_LEVEL,
+			   "Ignoring request: arguments do not match any credentials"));
+	  str = "";
 	}
 
 	printf("%s", str);
@@ -432,9 +438,7 @@ main(int argc, char **argv)
 	goto fail;
   }
 
-  emit_xml = test_emit_xml_format();
-
-  if (emit_xml) {
+  if (test_emit_xml_format()) {
 	emit_xml_header(stdout, "dacs_select_credentials");
 	printf("<dacs_select_credentials>\n");
 	printf("<ok>\n");
@@ -456,6 +460,29 @@ main(int argc, char **argv)
 	printf("</dacs_select_credentials>\n");
 	emit_xml_trailer(stdout);
   }
+  else if (test_emit_format(EMIT_FORMAT_JSON)) {
+	emit_json_header(stdout, "dacs_select_credentials");
+	printf("{ \"ok\":");
+
+	if (select_unauth)
+	  printf(" { \"unauth\":true }");
+	else if (nselected == 0)
+	  printf(" { }");
+	else {
+	  for (cr = credentials; cr != NULL; cr = cr->next) {
+		if (cr != credentials)
+		  printf(",");
+		if (cr->selected)
+		  printf(" { \"selected\": { ");
+		else
+		  printf(" { \"deselected\": { ");
+		printf("\"federation\":\"%s\", \"jurisdiction\":\"%s\", \"username\":\"%s\" }}",
+			   cr->federation, cr->home_jurisdiction, cr->username);
+	  }
+	}
+	printf("}}\n");
+	emit_json_trailer(stdout);
+  }
   else {
 	if (conf_val(CONF_CSS_PATH) != NULL)
 	  hc->css = ds_xprintf("%s/dacs_select_credentials.css",
