@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -11,10 +11,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2016\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: ssllib.c 2586 2012-03-15 16:21:40Z brachman $";
+  "$Id: ssllib.c 2912 2016-10-18 19:54:07Z brachman $";
 #endif
 
 #include "dacs_ssl.h"
@@ -49,7 +49,11 @@ seed_prng(char *rfile)
 int
 ssl_verify_callback(int ok, X509_STORE_CTX *ctx)
 {
+#if (OPENSSL_VERSION_NUMBER >= 0x1010000fL)
+  X509_STORE *store = X509_STORE_CTX_get0_store(ctx);
+#else
   X509_STORE *store = ctx->ctx;
+#endif
   Ssl_conf *conf;
 
   /*
@@ -228,15 +232,19 @@ ssl_post_connection_check(SSL *ssl, char *peer, Ssl_conf *conf)
 		ext_str = NULL;
 		if ((meth = X509V3_EXT_get(ext)) == NULL)
 		  break;
-		data = ext->value->data;
 
-#if (OPENSSL_VERSION_NUMBER > 0x00907000L)
+#if (OPENSSL_VERSION_NUMBER >= 0x1010000fL)
+		data = NULL;
+		ext_str = X509V3_EXT_d2i(ext);
+#elif (OPENSSL_VERSION_NUMBER > 0x00907000L)
+		data = ext->value->data;
 		if (meth->it)
 		  ext_str = ASN1_item_d2i(NULL, &data, ext->value->length,
 								  ASN1_ITEM_ptr(meth->it));
 		else
 		  ext_str = meth->d2i(NULL, &data, ext->value->length);
 #else
+		data = ext->value->data;
 		ext_str = meth->d2i(NULL, &data, ext->value->length);
 #endif
 		val = meth->i2v(meth, ext_str, NULL);
@@ -444,7 +452,7 @@ ssl_get_error_messages(void)
  * Return that context, or NULL on error.
  */
 SSL_CTX *
-ssl_setup_client_ctx(Ssl_conf *conf)
+ssl_setup_client_ctx(Ssl_conf *conf, char *servername)
 {
   SSL_CTX *ctx;
   const SSL_METHOD *m;
@@ -494,6 +502,10 @@ ssl_setup_client_ctx(Ssl_conf *conf)
 	}
   }
 
+  /*
+   * Disallow SSLv2 and earlier due to security defects.
+   * This allows only SSLv3 and TLSv1.
+   */
   SSL_CTX_set_options(ctx, SSL_OP_ALL | SSL_OP_NO_SSLv2);
 
   if (SSL_CTX_set_cipher_list(ctx, conf->cipher_list) != 1) {
@@ -510,6 +522,28 @@ ssl_setup_client_ctx(Ssl_conf *conf)
 	return(NULL);
   }
 
+  /*
+   * SNI - Server Name Indication (RFC 6066, S3)
+   * Testing service at:
+   *   https://sni.velox.ch/
+   */
+#ifdef OPENSSL_NO_TLSEXT
+  if (conf->use_sni && servername != NULL)
+	log_msg((LOG_WARN_LEVEL, "OpenSSL TLS extensions are unavailable"));
+#else
+  if (conf->use_sni && servername != NULL) {
+	if (SSL_set_tlsext_host_name(conf->ssl, servername) == 0) {
+	  log_msg((LOG_ERROR_LEVEL, "error setting SNI servername to \"%s\"",
+			   servername));
+	  log_msg((LOG_ERROR_LEVEL, "%s", ssl_get_error_messages()));
+	}
+	else
+	  log_msg((LOG_TRACE_LEVEL, "set SNI servername to \"%s\"", servername));
+  }
+else
+  log_msg((LOG_TRACE_LEVEL, "SNI available but not requested"));
+#endif
+
   SSL_CTX_set_verify(ctx, conf->verify_type, ssl_verify_callback);
   if (conf->verify_depth > 0)
 	SSL_CTX_set_verify_depth(ctx, conf->verify_depth);
@@ -566,7 +600,7 @@ ssl_transfer(char *hostname, char *port, Ssl_conf *conf,
   }
 
   ssl_init(conf);
-  if ((ctx = ssl_setup_client_ctx(conf)) == NULL)
+  if ((ctx = ssl_setup_client_ctx(conf, hostname)) == NULL)
 	return(-1);
   ssl = conf->ssl;
  
@@ -1113,6 +1147,11 @@ ssl_init_defaults(Ssl_conf *c)
   conf->cipher_list = DEFAULT_CIPHER_LIST;
   conf->rand_seed_file = DEFAULT_RAND_SEED_FILE;
   conf->peer_match_vec = NULL;
+#ifdef OPENSSL_NO_TLSEXT
+  conf->use_sni = 0;
+#else
+  conf->use_sni = 1;
+#endif
   conf->buffer_output = 0;
 
   return(conf);
