@@ -34,7 +34,7 @@ static const char copyright[] =
 "Copyright (c) 2003-2012\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: transform.c 2594 2012-10-19 17:28:49Z brachman $";
+  "$Id: transform.c 2861 2015-12-28 22:17:08Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -203,8 +203,7 @@ is_regex_directive(char *line, regex_t *regex, char **startp, char **endp,
 	if (errmsg != NULL) {
 	  errbuf[0] = '\0';
 	  regerror(st, regex, errbuf, sizeof(errbuf));
-	  *errmsg = ds_xprintf("bad regular expression: \"%s\": %s",
-						   regex, errbuf);
+	  *errmsg = ds_xprintf("bad regular expression: %s", errbuf);
 	}
 	rc = -1;
   }
@@ -1293,7 +1292,7 @@ transform(Transform_config *tc, char *object_name, char *idents, char **errmsg)
 	  case D_ERROR:
 		/* An invalid directive. */
 		if (errmsg != NULL)
-		  *errmsg = ds_xprintf("Line %d: %s", tc->linenum, errmsg);
+		  *errmsg = ds_xprintf("Line %d: %s", tc->linenum, *errmsg);
 		goto fail;
 		/*NOTREACHED*/
 	  }
