@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2012
+ * Copyright (c) 2003-2016
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -22,10 +22,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2012\n\
+"Copyright (c) 2003-2016\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: var.c 2594 2012-10-19 17:28:49Z brachman $";
+  "$Id: var.c 2913 2016-10-18 19:55:39Z brachman $";
 #endif
 
 static const char *log_module_name = "var";
@@ -378,8 +378,8 @@ var_value(char *str, Var **varp, char **endp, Var_resolve resolve,
  * If an error occurs, NULL is returned.
  * The callback function RESOLVE is invoked to determine the string value
  * of each variable reference.  It is called with the variable
- * and RESOLVE_ARG; it
- * returns the value of the variable or NULL if an error occurs.
+ * and RESOLVE_ARG; it returns the value of the variable or NULL if an error
+ * occurs.
  */
 Ds *
 var_expand(char *str, Var_resolve resolve, void *resolve_arg)
@@ -424,6 +424,20 @@ var_expand(char *str, Var_resolve resolve, void *resolve_arg)
 		}
 		ds_concatc(ds, cc);
 	  }
+	  else if (*p == 'x') {
+		p++;
+		if (!is_hexdigit((int) *p)) {
+		  log_msg((LOG_ERROR_LEVEL, "Invalid hex digit: %s", str));
+		  return(NULL);
+		}
+		cc = hexdigit_val((int) *p);
+		p++;
+		if (is_hexdigit(*p)) {
+		  cc = cc * 16 + hexdigit_val((int) *p);
+		  p++;
+		}
+		ds_concatc(ds, cc);
+	  }
 	  else {
 		/* Harbison & Steele 2.7.6 */
 		switch ((int) *p) {
@@ -599,7 +613,7 @@ var_ns_create(char *ns, Kwv *kwv)
 {
   char *n;
   Var_ns *v;
-  extern int acs_is_readonly_namespace(char *ns);
+  extern int acs_is_readonly_namespace(Var_ns *ns, char *name);
 
   if (ns == NULL) {
 #ifdef DEFAULT_NAMESPACE
@@ -619,7 +633,7 @@ var_ns_create(char *ns, Kwv *kwv)
   v = ALLOC(Var_ns);
 
   v->flags = VAR_NS_DEFAULT;
-  if (acs_is_readonly_namespace(ns))
+  if (acs_is_readonly_namespace(NULL, ns))
 	v->flags |= VAR_NS_READONLY;
 
   v->ns = strdup(n);
@@ -662,7 +676,8 @@ var_ns_new(Var_ns **root, char *name, Kwv *kwv)
   Var_ns *v;
 
   if (*root != NULL && (v = lookup(*root, name)) != NULL) {
-	if (v->kwv != NULL)
+	/* Be careful not to zap it accidentally. */
+	if (v->kwv != NULL && v->kwv != kwv)
 	  kwv_free(v->kwv);
 	v->kwv = kwv;
 
@@ -683,7 +698,8 @@ static Var_ns *
 do_delete(Var_ns **root, Var_ns *v)
 {
 
-  /* kwv_delete(v->kwv); */
+  kwv_delete(v->kwv, NULL);
+
   if (v->prev != NULL)
 	(v->prev)->next = v->next;
   else
