@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2013
+ * Copyright (c) 2003-2015
  * Distributed Systems Software.  All rights reserved.
  * See the file LICENSE for redistribution information.
  */
@@ -14,10 +14,10 @@
 
 #ifndef lint
 static const char copyright[] =
-"Copyright (c) 2003-2013\n\
+"Copyright (c) 2003-2015\n\
 Distributed Systems Software.  All rights reserved.";
 static const char revid[] =
-  "$Id: version.c 2646 2013-02-27 20:10:40Z brachman $";
+  "$Id: version.c 2842 2015-08-24 22:42:57Z brachman $";
 #endif
 
 #include "dacs.h"
@@ -114,10 +114,21 @@ dacs_component_versions(void)
 #endif
 
 #ifdef ENABLE_GDBM
-  {
-	extern char *gdbm_version;
+#include <gdbm.h>
 
+  {
+
+#if defined(GDBM_VERSION_MAJOR) && defined(GDBM_VERSION_MINOR)
+#ifdef GDBM_VERSION_PATCH
+	str = ds_xprintf("gdbm %d.%d-pl%d",
+					 GDBM_VERSION_MAJOR, GDBM_VERSION_MINOR,
+					 GDBM_VERSION_PATCH);
+#else
+	str = ds_xprintf("gdbm %d.%d", GDBM_VERSION_MAJOR, GDBM_VERSION_MINOR);
+#endif
+#else
 	str = ds_xprintf("%s", gdbm_version);
+#endif
 	dsvec_add_ptr(dsv, str);
   }
 #endif
@@ -249,18 +260,30 @@ static struct nsym {
 #ifdef ENABLE_USER_INFO
   NSYM(ENABLE_USER_INFO),
 #endif
+#ifdef ENABLE_HUSH_STARTUP_LOGGING
+  NSYM(ENABLE_HUSH_STARTUP_LOGGING),
+#endif
 #ifdef ENABLE_ACCESS_TOKENS
   NSYM(ENABLE_ACCESS_TOKENS),
 #endif
 #ifdef ENABLE_RULE_PATTERNS
   NSYM(ENABLE_RULE_PATTERNS),
 #endif
+#ifdef ENABLE_ADDONS
+  NSYM(ENABLE_ADDONS),
+#endif
 #ifdef _BSD_SOURCE
   NSYM(_BSD_SOURCE),
 #endif
+#ifdef _GNU_SOURCE
+  NSYM(_GNU_SOURCE),
+#endif
 #ifdef _ISOC99_SOURCE
   NSYM(_ISOC99_SOURCE),
 #endif
+#ifdef HAVE_LIBPAM
+  NSYM(HAVE_LIBPAM),
+#endif
 #ifdef HAVE_READLINE
   NSYM(HAVE_READLINE),
 #endif
@@ -272,6 +295,12 @@ static struct nsym {
 #endif
 #ifdef HAVE_SHM_OPEN
   NSYM(HAVE_SHM_OPEN),
+#endif
+#ifdef DSS_HOSTED
+  NSYM(DSS_HOSTED),
+#endif
+#ifdef ENABLE_DEVELOPER
+  NSYM(ENABLE_DEVELOPER),
 #endif
   { NULL, 0 }
 };
@@ -363,7 +392,7 @@ fmt_row(FILE *fp, int row, char *name, char *value)
 int
 dacsversion_main(int argc, char **argv, int do_init, void *main_out)
 {
-  int i, emit_xml, xargc, rc;
+  int i, xargc, rc;
   char **xargv, *errmsg, *p, *remote_addr, *versions;
   DACS_app_type app_type;
   Dsvec *conf_dsv, *dsv;
@@ -378,13 +407,11 @@ dacsversion_main(int argc, char **argv, int do_init, void *main_out)
   if ((remote_addr = getenv("REMOTE_ADDR")) == NULL) {
 	app_type = DACS_STANDALONE;
 	log_module_name = "dacsversion";
-	emit_xml = 0;
 	rc = 0;
   }
   else {
 	app_type = DACS_WEB_SERVICE;
 	log_module_name = "dacs_version";
-	emit_xml = 1;
 	rc = 1;
   }
 
@@ -413,26 +440,46 @@ dacsversion_main(int argc, char **argv, int do_init, void *main_out)
 	}
   }
 
-  emit_xml = test_emit_xml_format();
-
   rc = 0;
 
  fail:
-  if (emit_xml) {
+  if (test_emit_xml_format()) {
 	emit_xml_header(stdout, "dacs_version");
 	printf("<%s", make_xml_root_element("dacs_version"));
 	printf(" number=\"%s\"\n", DACS_VERSION_NUMBER);
 	printf(" release=\"%s\"\n", DACS_VERSION_RELEASE);
 	printf(" date=\"%s\"\n", DACS_VERSION_DATE);
 	printf(" revid=\"%s\"\n", DACS_VERSION_REVID);
-	printf(" build_os=\"%s\"\n", dacs_build_os_string());
-	printf(" exec_os=\"%s\"\n", dacs_runtime_os_string());
+	printf(" build_platform=\"%s\"\n", dacs_build_os_string());
+	printf(" exec_platform=\"%s\"\n", dacs_runtime_os_string());
+	printf(" digests=\"%s\"\n", crypto_digest_list());
 	printf(" other=\"%s\">\n", versions);
 	for (i = 0; dsv != NULL && (p = dsvec_ptr_index(dsv, i)) != NULL; i++)
 	  printf(" <file revid=\"%s\"/>\n", p);
 	printf("</dacs_version>\n");
 	emit_xml_trailer(stdout);
   }
+  else if (test_emit_format(EMIT_FORMAT_JSON)) {
+    emit_json_header(stdout, "dacs_version");
+
+	printf("\"number\":\"%s\"", DACS_VERSION_NUMBER);
+	printf(", \"release\":\"%s\"", DACS_VERSION_RELEASE);
+	printf(", \"date\":\"%s\"", DACS_VERSION_DATE);
+	printf(", \"revid\":\"%s\"", DACS_VERSION_REVID);
+	printf(", \"build_platform\":\"%s\"", dacs_build_os_string());
+	printf(", \"exec_platform\":\"%s\"", dacs_runtime_os_string());
+	printf(", \"digests\":\"%s\"", crypto_digest_list());
+	printf(", \"other\":\"%s\",\n", versions);
+
+	printf("\"file\": [");
+	for (i = 0; dsv != NULL && (p = dsvec_ptr_index(dsv, i)) != NULL; i++) {
+	  printf("{ \"revid\":\"%s\" }%s\n",
+			 p, (i == (dsvec_len(dsv) - 1)) ? "" : ",");
+	}
+	printf("]}\n");
+
+	emit_json_trailer(stdout);
+  }
   else if (test_emit_format(EMIT_FORMAT_HTML)) {
 	if (conf_val(CONF_CSS_PATH) != NULL)
 	  hc->css = ds_xprintf("%s/dacs_version.css", conf_val(CONF_CSS_PATH));
@@ -462,6 +509,7 @@ dacsversion_main(int argc, char **argv, int do_init, void *main_out)
 	  html_cell2(tab, "revid", DACS_VERSION_REVID);
 	  html_cell2(tab, "build_platform", dacs_build_os_string());
 	  html_cell2(tab, "exec_platform", dacs_runtime_os_string());
+	  html_cell2(tab, "digests", crypto_digest_list());
 	  html_cell2(tab, "other", versions);
 
 	  for (i = 0; dsv != NULL && (p = dsvec_ptr_index(dsv, i)) != NULL; i++)
@@ -491,8 +539,12 @@ dacsversion_main(int argc, char **argv, int do_init, void *main_out)
 	  printf("release=\"%s\"\n", DACS_VERSION_RELEASE);
 	  printf("date=\"%s\"\n", DACS_VERSION_DATE);
 	  printf("revid=\"%s\"\n", DACS_VERSION_REVID);
+#ifdef ENABLE_ADDONS
+	  printf("addons=\"enabled\"\n");
+#endif
 	  printf("build_platform=\"%s\"\n", dacs_build_os_string());
 	  printf("exec_platform=\"%s\"\n", dacs_runtime_os_string());
+	  printf("digests=\"%s\"\n", crypto_digest_list());
 	  printf("other=\"%s\"\n", versions);
 	  if (verbose_level > 1) {
 		for (i = 0; dsv != NULL && (p = dsvec_ptr_index(dsv, i)) != NULL; i++)
